@@ -1,4 +1,3 @@
-# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
 # Copyright 2023 The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,33 +17,36 @@ import paddle
 
 from ...models.unet_1d import UNet1DModel
 from ...pipelines import DiffusionPipeline
-from ...utils import randn_tensor
 from ...utils.dummy_paddle_objects import DDPMScheduler
+from ...utils.paddle_utils import randn_tensor
 
 
 class ValueGuidedRLPipeline(DiffusionPipeline):
     r"""
-    This model inherits from [`DiffusionPipeline`]. Check the superclass documentation for the generic methods the
-    library implements for all the pipelines (such as downloading or saving, running on a particular device, etc.)
-    Pipeline for sampling actions from a diffusion model trained to predict sequences of states.
+    Pipeline for value-guided sampling from a diffusion model trained to predict sequences of states.
 
-    Original implementation inspired by this repository: https://github.com/jannerm/diffuser.
+    This model inherits from [`DiffusionPipeline`]. Check the superclass documentation for the generic methods
+    implemented for all pipelines (downloading, saving, running on a particular device, etc.).
 
     Parameters:
-        value_function ([`UNet1DModel`]): A specialized UNet for fine-tuning trajectories base on reward.
-        unet ([`UNet1DModel`]): U-Net architecture to denoise the encoded trajectories.
+        value_function ([`UNet1DModel`]):
+            A specialized UNet for fine-tuning trajectories base on reward.
+        unet ([`UNet1DModel`]):
+            UNet architecture to denoise the encoded trajectories.
         scheduler ([`SchedulerMixin`]):
             A scheduler to be used in combination with `unet` to denoise the encoded trajectories. Default for this
             application is [`DDPMScheduler`].
-        env: An environment following the OpenAI gym API to act in. For now only Hopper has pretrained models.
+        env ():
+            An environment following the OpenAI gym API to act in. For now only Hopper has pretrained models.
     """
 
     def __init__(
-            self,
-            value_function: UNet1DModel,
-            unet: UNet1DModel,
-            scheduler: DDPMScheduler,
-            env, ):
+        self,
+        value_function: UNet1DModel,
+        unet: UNet1DModel,
+        scheduler: DDPMScheduler,
+        env,
+    ):
         super().__init__()
         self.value_function = value_function
         self.unet = unet
@@ -73,7 +75,7 @@ class ValueGuidedRLPipeline(DiffusionPipeline):
         return x_in * self.stds[key] + self.means[key]
 
     def to_paddle(self, x_in):
-        if type(x_in) is dict:
+        if isinstance(x_in, dict):
             return {k: self.to_paddle(v) for k, v in x_in.items()}
         elif paddle.is_tensor(x_in):
             return x_in
@@ -89,14 +91,13 @@ class ValueGuidedRLPipeline(DiffusionPipeline):
         y = None
         for i in self.progress_bar(self.scheduler.timesteps):
             # create batch of timesteps to pass into model
-            timesteps = paddle.full((batch_size, ), i, dtype=paddle.int64)
+            timesteps = paddle.full((batch_size,), i, dtype=paddle.int64)
             for _ in range(n_guide_steps):
                 with paddle.set_grad_enabled(True):
                     x.stop_gradient = False
 
                     # permute to match dimension for pre-trained models
-                    y = self.value_function(x.transpose([0, 2, 1]),
-                                            timesteps).sample
+                    y = self.value_function(x.transpose([0, 2, 1]), timesteps).sample
                     grad = paddle.autograd.grad([y.sum()], [x])[0]
 
                     posterior_variance = self.scheduler._get_variance(i)
@@ -108,24 +109,17 @@ class ValueGuidedRLPipeline(DiffusionPipeline):
                 x = x + scale * grad
                 x = self.reset_x0(x, conditions, self.action_dim)
 
-            prev_x = self.unet(x.transpose([0, 2, 1]),
-                               timesteps).sample.transpose([0, 2, 1])
+            prev_x = self.unet(x.transpose([0, 2, 1]), timesteps).sample.transpose([0, 2, 1])
 
             # TODO: verify deprecation of this kwarg
-            x = self.scheduler.step(
-                prev_x, i, x, predict_epsilon=False)["prev_sample"]
+            x = self.scheduler.step(prev_x, i, x, predict_epsilon=False)["prev_sample"]
 
             # apply conditions to the trajectory (set the initial state)
             x = self.reset_x0(x, conditions, self.action_dim)
             x = self.to_paddle(x)
         return x, y
 
-    def __call__(self,
-                 obs,
-                 batch_size=64,
-                 planning_horizon=32,
-                 n_guide_steps=2,
-                 scale=0.1):
+    def __call__(self, obs, batch_size=64, planning_horizon=32, n_guide_steps=2, scale=0.1):
         # normalize the observations and create  batch dimension
         obs = self.normalize(obs, "observations")
         obs = obs[None].repeat(batch_size, axis=0)
@@ -144,7 +138,7 @@ class ValueGuidedRLPipeline(DiffusionPipeline):
         # sort output trajectories by value
         sorted_idx = paddle.argsort(y, 0, descending=True).squeeze()
         sorted_values = x[sorted_idx]
-        actions = sorted_values[:, :, :self.action_dim]
+        actions = sorted_values[:, :, : self.action_dim]
         actions = actions.detach().cpu().numpy()
         denorm_actions = self.de_normalize(actions, key="actions")
 
