@@ -1,4 +1,3 @@
-# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
 # Copyright 2023 The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -47,35 +46,40 @@ class DualTransformer2DModel(nn.Layer):
     """
 
     def __init__(
-            self,
-            num_attention_heads: int=16,
-            attention_head_dim: int=88,
-            in_channels: Optional[int]=None,
-            num_layers: int=1,
-            dropout: float=0.0,
-            norm_num_groups: int=32,
-            cross_attention_dim: Optional[int]=None,
-            attention_bias: bool=False,
-            sample_size: Optional[int]=None,
-            num_vector_embeds: Optional[int]=None,
-            activation_fn: str="geglu",
-            num_embeds_ada_norm: Optional[int]=None, ):
+        self,
+        num_attention_heads: int = 16,
+        attention_head_dim: int = 88,
+        in_channels: Optional[int] = None,
+        num_layers: int = 1,
+        dropout: float = 0.0,
+        norm_num_groups: int = 32,
+        cross_attention_dim: Optional[int] = None,
+        attention_bias: bool = False,
+        sample_size: Optional[int] = None,
+        num_vector_embeds: Optional[int] = None,
+        activation_fn: str = "geglu",
+        num_embeds_ada_norm: Optional[int] = None,
+    ):
         super().__init__()
-        self.transformers = nn.LayerList([
-            Transformer2DModel(
-                num_attention_heads=num_attention_heads,
-                attention_head_dim=attention_head_dim,
-                in_channels=in_channels,
-                num_layers=num_layers,
-                dropout=dropout,
-                norm_num_groups=norm_num_groups,
-                cross_attention_dim=cross_attention_dim,
-                attention_bias=attention_bias,
-                sample_size=sample_size,
-                num_vector_embeds=num_vector_embeds,
-                activation_fn=activation_fn,
-                num_embeds_ada_norm=num_embeds_ada_norm, ) for _ in range(2)
-        ])
+        self.transformers = nn.LayerList(
+            [
+                Transformer2DModel(
+                    num_attention_heads=num_attention_heads,
+                    attention_head_dim=attention_head_dim,
+                    in_channels=in_channels,
+                    num_layers=num_layers,
+                    dropout=dropout,
+                    norm_num_groups=norm_num_groups,
+                    cross_attention_dim=cross_attention_dim,
+                    attention_bias=attention_bias,
+                    sample_size=sample_size,
+                    num_vector_embeds=num_vector_embeds,
+                    activation_fn=activation_fn,
+                    num_embeds_ada_norm=num_embeds_ada_norm,
+                )
+                for _ in range(2)
+            ]
+        )
 
         # Variables that can be set by a pipeline:
 
@@ -91,25 +95,33 @@ class DualTransformer2DModel(nn.Layer):
         self.transformer_index_for_condition = [1, 0]
 
     def forward(
-            self,
-            hidden_states,
-            encoder_hidden_states,
-            timestep=None,
-            attention_mask=None,
-            cross_attention_kwargs=None,
-            return_dict: bool=True, ):
+        self,
+        hidden_states,
+        encoder_hidden_states,
+        timestep=None,
+        added_cond_kwargs=None,
+        class_labels=None,
+        cross_attention_kwargs=None,
+        attention_mask=None,
+        encoder_attention_mask=None,
+        return_dict: bool = True,
+    ):
         """
         Args:
             hidden_states ( When discrete, `paddle.Tensor` of shape `(batch size, num latent pixels)`.
                 When continuous, `paddle.Tensor` of shape `(batch size, channel, height, width)`): Input
-                hidden_states
+                hidden_states.
             encoder_hidden_states ( `paddle.Tensor` of shape `(batch size, encoder_hidden_states dim)`, *optional*):
                 Conditional embeddings for cross attention layer. If not given, cross-attention defaults to
                 self-attention.
             timestep ( `paddle.Tensor`, *optional*):
                 Optional timestep to be applied as an embedding in AdaLayerNorm's. Used to indicate denoising step.
             attention_mask (`paddle.Tensor`, *optional*):
-                Optional attention mask to be applied in Attention
+                Optional attention mask to be applied in Attention.
+            cross_attention_kwargs (`dict`, *optional*):
+                A kwargs dictionary that if specified is passed along to the `AttentionProcessor` as defined under
+                `self.processor` in
+                [diffusers.models.attention_processor](https://github.com/huggingface/diffusers/blob/main/src/diffusers/models/attention_processor.py).
             return_dict (`bool`, *optional*, defaults to `True`):
                 Whether or not to return a [`models.unet_2d_condition.UNet2DConditionOutput`] instead of a plain tuple.
 
@@ -125,23 +137,22 @@ class DualTransformer2DModel(nn.Layer):
         # attention_mask is not used yet
         for i in range(2):
             # for each of the two transformers, pass the corresponding condition tokens
-            condition_state = encoder_hidden_states[:, tokens_start:tokens_start
-                                                    + self.condition_lengths[i]]
+            condition_state = encoder_hidden_states[:, tokens_start : tokens_start + self.condition_lengths[i]]
             transformer_index = self.transformer_index_for_condition[i]
             encoded_state = self.transformers[transformer_index](
                 input_states,
                 encoder_hidden_states=condition_state,
                 timestep=timestep,
                 cross_attention_kwargs=cross_attention_kwargs,
-                return_dict=False, )[0]
+                return_dict=False,
+            )[0]
             encoded_states.append(encoded_state - input_states)
             tokens_start += self.condition_lengths[i]
 
-        output_states = encoded_states[0] * self.mix_ratio + encoded_states[
-            1] * (1 - self.mix_ratio)
+        output_states = encoded_states[0] * self.mix_ratio + encoded_states[1] * (1 - self.mix_ratio)
         output_states = output_states + input_states
 
         if not return_dict:
-            return (output_states, )
+            return (output_states,)
 
         return Transformer2DModelOutput(sample=output_states)
