@@ -19,74 +19,71 @@
 # Adapted from https://github.com/dbolya/tomesd
 
 import math
-from typing import Any, Callable, Dict, Tuple, Type, Union
+from typing import Any, Callable, Dict, Optional, Tuple, Type, Union
 
 import paddle
 import paddle.nn as nn
 
-from ..models.transformer_2d import BasicTransformerBlock
+from ..models.attention import BasicTransformerBlock, _chunked_feed_forward
 from ..pipelines.pipeline_utils import DiffusionPipeline
-from .ppnlp_patch_utils import patch_to
+from .paddle_patch import patch_to
 
 TOME_PREFIX = "ToMe"
 
 
 def scatter_reduce(
-        input: paddle.Tensor,
-        dim: int,
-        index: paddle.Tensor,
-        src: paddle.Tensor,
-        reduce: str="mean",
-        include_self: bool=True, ) -> paddle.Tensor:
+    input: paddle.Tensor,
+    dim: int,
+    index: paddle.Tensor,
+    src: paddle.Tensor,
+    reduce: str = "mean",
+    include_self: bool = True,
+) -> paddle.Tensor:
     # reduce "sum", "prod", "mean",
     # TODO support "amax", "amin" and include_self = False
     if reduce in ["sum", "assign", "add"]:
         if reduce == "sum":
             reduce = "add"
-        input.put_along_axis_(
-            indices=index, values=src, axis=dim, reduce=reduce)
+        input.put_along_axis_(indices=index, values=src, axis=dim, reduce=reduce)
     elif reduce == "mean":
         # compute sum first
         input.put_along_axis_(indices=index, values=src, axis=dim, reduce="add")
         # compute div secondly
         input_div = paddle.ones_like(input).put_along_axis(
             indices=index,
-            values=paddle.to_tensor(
-                1.0, dtype=input.dtype),
+            values=paddle.to_tensor(1.0, dtype=input.dtype),
             axis=dim,
-            reduce="add")
+            reduce="add",
+        )
         input = input / input_div
     elif reduce in ["prod", "mul", "multiply"]:
-        input = paddle.put_along_axis(
-            input.cpu(),
-            indices=index.cpu(),
-            values=src.cpu(),
-            axis=dim,
-            reduce="mul")._to(device=paddle.get_device())
-    else:
-        raise NotImplementedError(
-            "only support mode in ['add', 'sum', 'prod', 'mul', 'multiply', 'mean', 'assign']!"
+        input = paddle.put_along_axis(input.cpu(), indices=index.cpu(), values=src.cpu(), axis=dim, reduce="mul")._to(
+            device=paddle.get_device()
         )
+    else:
+        raise NotImplementedError("only support mode in ['add', 'sum', 'prod', 'mul', 'multiply', 'mean', 'assign']!")
     return input
 
 
-# patch scatter_reduce
-paddle.scatter_reduce = scatter_reduce
-paddle.Tensor.scatter_reduce = scatter_reduce
+if not hasattr(paddle, "scatter_reduce"):
+    # patch scatter_reduce
+    paddle.scatter_reduce = scatter_reduce
+    paddle.Tensor.scatter_reduce = scatter_reduce
 
 
-def do_nothing(x: paddle.Tensor, mode: str=None):
+def do_nothing(x: paddle.Tensor, mode: str = None):
     return x
 
 
 def bipartite_soft_matching_random2d(
-        metric: paddle.Tensor,
-        w: int,
-        h: int,
-        sx: int,
-        sy: int,
-        r: int,
-        no_rand: bool=False, ) -> Tuple[Callable, Callable]:
+    metric: paddle.Tensor,
+    w: int,
+    h: int,
+    sx: int,
+    sy: int,
+    r: int,
+    no_rand: bool = False,
+) -> Tuple[Callable, Callable]:
     """
     Partitions the tokens into src and dst and merges r tokens from src to dst.
     Dst tokens are partitioned by choosing one randomy in each (sx, sy) region.
@@ -112,24 +109,23 @@ def bipartite_soft_matching_random2d(
         if no_rand:
             rand_idx = paddle.zeros((hsy, wsx, 1), dtype=paddle.int64)
         else:
-            rand_idx = paddle.randint(
-                sy * sx, shape=(hsy, wsx, 1), dtype=paddle.int64)
+            rand_idx = paddle.randint(sy * sx, shape=(hsy, wsx, 1), dtype=paddle.int64)
 
         # The image might not divide sx and sy, so we need to work on a view of the top left if the idx buffer instead
         idx_buffer_view = paddle.zeros([hsy, wsx, sy * sx], dtype=paddle.int64)
         idx_buffer_view.put_along_axis_(
             axis=2,
             indices=rand_idx,
-            values=-paddle.ones_like(
-                rand_idx, dtype=rand_idx.dtype))
+            values=-paddle.ones_like(rand_idx, dtype=rand_idx.dtype),
+        )
         idx_buffer_view = (
-            idx_buffer_view.reshape([hsy, wsx, sy, sx]).transpose(
-                [0, 2, 1, 3]).reshape([hsy * sy, wsx * sx]))
+            idx_buffer_view.reshape([hsy, wsx, sy, sx]).transpose([0, 2, 1, 3]).reshape([hsy * sy, wsx * sx])
+        )
 
         # Image is not divisible by sx or sy so we need to move it into a new buffer
         if (hsy * sy) < h or (wsx * sx) < w:
             idx_buffer = paddle.zeros([h, w], dtype=paddle.int64)
-            idx_buffer[:(hsy * sy), :(wsx * sx)] = idx_buffer_view
+            idx_buffer[: (hsy * sy), : (wsx * sx)] = idx_buffer_view
         else:
             idx_buffer = idx_buffer_view
 
@@ -147,10 +143,8 @@ def bipartite_soft_matching_random2d(
         def split(x):
             C = x.shape[-1]
 
-            src = x.take_along_axis(
-                indices=a_idx.expand([B, N - num_dst, C]), axis=1)
-            dst = x.take_along_axis(
-                indices=b_idx.expand([B, num_dst, C]), axis=1)
+            src = x.take_along_axis(indices=a_idx.expand([B, N - num_dst, C]), axis=1)
+            dst = x.take_along_axis(indices=b_idx.expand([B, num_dst, C]), axis=1)
             return src, dst
 
         # Cosine similarity between A and B
@@ -178,12 +172,10 @@ def bipartite_soft_matching_random2d(
         src, dst = split(x)
         n, t1, c = src.shape
 
-        unm = src.take_along_axis(
-            indices=unm_idx.expand([n, t1 - r, c]), axis=-2)
+        unm = src.take_along_axis(indices=unm_idx.expand([n, t1 - r, c]), axis=-2)
         src = src.take_along_axis(indices=src_idx.expand([n, r, c]), axis=-2)
 
-        dst = scatter_reduce(
-            dst, -2, dst_idx.expand([n, r, c]), src, reduce=mode)
+        dst = scatter_reduce(dst, -2, dst_idx.expand([n, r, c]), src, reduce=mode)
 
         return paddle.concat([unm, dst], axis=1)
 
@@ -200,25 +192,27 @@ def bipartite_soft_matching_random2d(
         out.put_along_axis_(
             indices=b_idx.expand([B, num_dst, c]),
             values=dst,
-            axis=-2, )
+            axis=-2,
+        )
         out.put_along_axis_(
-            indices=a_idx.expand([B, a_idx.shape[1], 1]).take_along_axis(
-                indices=unm_idx, axis=1).expand([B, unm_len, c]),
+            indices=a_idx.expand([B, a_idx.shape[1], 1])
+            .take_along_axis(indices=unm_idx, axis=1)
+            .expand([B, unm_len, c]),
             values=unm,
-            axis=-2, )
+            axis=-2,
+        )
         out.put_along_axis_(
-            indices=a_idx.expand([B, a_idx.shape[1], 1]).take_along_axis(
-                indices=src_idx, axis=1).expand([B, r, c]),
+            indices=a_idx.expand([B, a_idx.shape[1], 1]).take_along_axis(indices=src_idx, axis=1).expand([B, r, c]),
             values=src,
-            axis=-2, )
+            axis=-2,
+        )
 
         return out
 
     return merge, unmerge
 
 
-def compute_merge(x: paddle.Tensor,
-                  tome_info: Dict[str, Any]) -> Tuple[Callable, ...]:
+def compute_merge(x: paddle.Tensor, tome_info: Dict[str, Any]) -> Tuple[Callable, ...]:
     original_h, original_w = tome_info["size"]
     original_tokens = original_h * original_w
     downsample = int(math.ceil(math.sqrt(original_tokens // x.shape[1])))
@@ -232,8 +226,7 @@ def compute_merge(x: paddle.Tensor,
         # If the batch size is odd, then it's not possible for promted and unprompted images to be in the same
         # batch, which causes artifacts with use_rand, so force it to be off.
         use_rand = False if x.shape[0] % 2 == 1 else args["use_rand"]
-        m, u = bipartite_soft_matching_random2d(x, w, h, args["sx"], args["sy"],
-                                                r, not use_rand)
+        m, u = bipartite_soft_matching_random2d(x, w, h, args["sx"], args["sy"], r, not use_rand)
     else:
         m, u = (do_nothing, do_nothing)
 
@@ -255,80 +248,134 @@ def make_tome_block(block_class: Type[nn.Layer]) -> Type[nn.Layer]:
         _parent = block_class
 
         def forward(
-                self: BasicTransformerBlock,
-                hidden_states,
-                attention_mask=None,
-                encoder_hidden_states=None,
-                encoder_attention_mask=None,
-                timestep=None,
-                cross_attention_kwargs=None,
-                class_labels=None, ) -> paddle.Tensor:
+            self: BasicTransformerBlock,
+            hidden_states: paddle.Tensor,
+            attention_mask: Optional[paddle.Tensor] = None,
+            encoder_hidden_states: Optional[paddle.Tensor] = None,
+            encoder_attention_mask: Optional[paddle.Tensor] = None,
+            timestep: Optional[paddle.Tensor] = None,
+            cross_attention_kwargs: Dict[str, Any] = None,
+            class_labels: Optional[paddle.Tensor] = None,
+        ) -> paddle.Tensor:
             # (1) ToMe
-            m_a, m_c, m_m, u_a, u_c, u_m = compute_merge(hidden_states,
-                                                         self._tome_info)
+            m_a, m_c, m_m, u_a, u_c, u_m = compute_merge(hidden_states, self._tome_info)
 
+            # Notice that normalization is always applied before the real computation in the following blocks.
+            # 1. Self-Attention
+            batch_size = hidden_states.shape[0]
             if self.use_ada_layer_norm:
                 norm_hidden_states = self.norm1(hidden_states, timestep)
             elif self.use_ada_layer_norm_zero:
                 norm_hidden_states, gate_msa, shift_mlp, scale_mlp, gate_mlp = self.norm1(
-                    hidden_states,
-                    timestep,
-                    class_labels,
-                    hidden_dtype=hidden_states.dtype)
-            else:
+                    hidden_states, timestep, class_labels, hidden_dtype=hidden_states.dtype
+                )
+            elif self.use_layer_norm:
                 norm_hidden_states = self.norm1(hidden_states)
+            elif self.use_ada_layer_norm_single:
+                shift_msa, scale_msa, gate_msa, shift_mlp, scale_mlp, gate_mlp = (
+                    self.scale_shift_table[None] + timestep.reshape([batch_size, 6, -1])
+                ).chunk(6, axis=1)
+                norm_hidden_states = self.norm1(hidden_states)
+                norm_hidden_states = norm_hidden_states * (1 + scale_msa) + shift_msa
+                norm_hidden_states = norm_hidden_states.squeeze(1)
+            else:
+                raise ValueError("Incorrect norm used")
+
+            if self.pos_embed is not None:
+                norm_hidden_states = self.pos_embed(norm_hidden_states)
 
             # (2) ToMe m_a
             norm_hidden_states = m_a(norm_hidden_states)
 
-            # 1. Self-Attention
-            cross_attention_kwargs = cross_attention_kwargs if cross_attention_kwargs is not None else {}
+            # 1. Retrieve lora scale.
+            lora_scale = cross_attention_kwargs.get("scale", 1.0) if cross_attention_kwargs is not None else 1.0
+
+            # 2. Prepare GLIGEN inputs
+            cross_attention_kwargs = cross_attention_kwargs.copy() if cross_attention_kwargs is not None else {}
+            gligen_kwargs = cross_attention_kwargs.pop("gligen", None)
+
             attn_output = self.attn1(
                 norm_hidden_states,
-                encoder_hidden_states=encoder_hidden_states
-                if self.only_cross_attention else None,
+                encoder_hidden_states=encoder_hidden_states if self.only_cross_attention else None,
                 attention_mask=attention_mask,
-                **cross_attention_kwargs, )
+                **cross_attention_kwargs,
+            )
             if self.use_ada_layer_norm_zero:
                 attn_output = gate_msa.unsqueeze(1) * attn_output
+            elif self.use_ada_layer_norm_single:
+                attn_output = gate_msa * attn_output
 
             # (3) ToMe u_a
             hidden_states = u_a(attn_output) + hidden_states
 
+            if hidden_states.ndim == 4:
+                hidden_states = hidden_states.squeeze(1)
+
+            # 2.5 GLIGEN Control
+            if gligen_kwargs is not None:
+                hidden_states = self.fuser(hidden_states, gligen_kwargs["objs"])
+
+            # 3. Cross-Attention
             if self.attn2 is not None:
-                norm_hidden_states = (self.norm2(hidden_states, timestep)
-                                      if self.use_ada_layer_norm else
-                                      self.norm2(hidden_states))
+                if self.use_ada_layer_norm:
+                    norm_hidden_states = self.norm2(hidden_states, timestep)
+                elif self.use_ada_layer_norm_zero or self.use_layer_norm:
+                    norm_hidden_states = self.norm2(hidden_states)
+                elif self.use_ada_layer_norm_single:
+                    # For PixArt norm2 isn't applied here:
+                    # https://github.com/PixArt-alpha/PixArt-alpha/blob/0f55e922376d8b797edd44d25d0e7464b260dcab/diffusion/model/nets/PixArtMS.py#L70C1-L76C103
+                    norm_hidden_states = hidden_states
+                else:
+                    raise ValueError("Incorrect norm")
+
                 # (4) ToMe m_c
                 norm_hidden_states = m_c(norm_hidden_states)
 
-                # 2. Cross-Attention
+                if self.pos_embed is not None and self.use_ada_layer_norm_single is False:
+                    norm_hidden_states = self.pos_embed(norm_hidden_states)
+
                 attn_output = self.attn2(
                     norm_hidden_states,
                     encoder_hidden_states=encoder_hidden_states,
                     attention_mask=encoder_attention_mask,
-                    **cross_attention_kwargs, )
+                    **cross_attention_kwargs,
+                )
                 # (5) ToMe u_c
                 hidden_states = u_c(attn_output) + hidden_states
 
-            # 3. Feed-forward
-            norm_hidden_states = self.norm3(hidden_states)
+            # 4. Feed-forward
+            if not self.use_ada_layer_norm_single:
+                norm_hidden_states = self.norm3(hidden_states)
 
             if self.use_ada_layer_norm_zero:
-                norm_hidden_states = norm_hidden_states * (
-                    1 + scale_mlp[:, None]) + shift_mlp[:, None]
+                norm_hidden_states = norm_hidden_states * (1 + scale_mlp[:, None]) + shift_mlp[:, None]
+
+            if self.use_ada_layer_norm_single:
+                norm_hidden_states = self.norm2(hidden_states)
+                norm_hidden_states = norm_hidden_states * (1 + scale_mlp) + shift_mlp
 
             # (6) ToMe m_m
             norm_hidden_states = m_m(norm_hidden_states)
 
-            ff_output = self.ff(norm_hidden_states)
+            if self._chunk_size is not None:
+                # "feed_forward_chunk_size" can be used to save memory
+                ff_output = _chunked_feed_forward(
+                    self.ff, norm_hidden_states, self._chunk_dim, self._chunk_size, lora_scale=lora_scale
+                )
+            else:
+                ff_output = self.ff(norm_hidden_states, scale=lora_scale)
 
             if self.use_ada_layer_norm_zero:
                 ff_output = gate_mlp.unsqueeze(1) * ff_output
+            elif self.use_ada_layer_norm_single:
+                ff_output = gate_mlp * ff_output
 
             # (7) ToMe u_m
             hidden_states = u_m(ff_output) + hidden_states
 
+            if hidden_states.ndim == 4:
+                hidden_states = hidden_states.squeeze(1)
+
             return hidden_states
 
     return ToMeBasicTransformerBlock
@@ -345,17 +392,16 @@ def hook_tome_model(model: nn.Layer):
 
 
 @patch_to([DiffusionPipeline, nn.Layer])
-def remove_tome(model_or_pipe: Union[nn.Layer, DiffusionPipeline],
-                only_return_self: bool=True):
+def remove_tome(model_or_pipe: Union[nn.Layer, DiffusionPipeline], only_return_self: bool = True):
     """Removes a patch from a ToMeXXX module if it was already patched."""
     model_list = []
     if isinstance(model_or_pipe, DiffusionPipeline):
-        for _, component in model_or_pipe.components.items():
-            if isinstance(component, nn.Layer):
+        for name, component in model_or_pipe.components.items():
+            if isinstance(component, nn.Layer) and name not in ["vqvae", "vae"]:
                 model_list.append(component)
             elif isinstance(component, (tuple, list)):
                 for each_component in component:
-                    if isinstance(component, nn.Layer):
+                    if isinstance(component, nn.Layer) and name not in ["vqvae", "vae"]:
                         model_list.append(each_component)
     elif isinstance(model_or_pipe, nn.Layer):
         model_list.append(model_or_pipe)
@@ -377,15 +423,16 @@ def remove_tome(model_or_pipe: Union[nn.Layer, DiffusionPipeline],
 
 @patch_to([DiffusionPipeline, nn.Layer])
 def apply_tome(
-        model_or_pipe: Union[nn.Layer, DiffusionPipeline],
-        ratio: float=0.5,
-        max_downsample: int=1,
-        sx: int=2,
-        sy: int=2,
-        use_rand: bool=True,
-        merge_attn: bool=True,
-        merge_crossattn: bool=False,
-        merge_mlp: bool=False, ):
+    model_or_pipe: Union[nn.Layer, DiffusionPipeline],
+    ratio: float = 0.5,
+    max_downsample: int = 1,
+    sx: int = 2,
+    sy: int = 2,
+    use_rand: bool = True,
+    merge_attn: bool = True,
+    merge_crossattn: bool = False,
+    merge_mlp: bool = False,
+):
     """
     Patches a stable diffusion model_or_pipe with ToMe.
     Apply this to the highest level stable diffusion object (i.e., it should have a .unet).
