@@ -19,22 +19,20 @@ import numpy as np
 import paddle
 import PIL
 
-from paddlenlp.transformers import CLIPImageProcessor, CLIPTokenizer
+from ppdiffusers.transformers import CLIPImageProcessor, CLIPTokenizer
 
-from ...pipeline_utils import DiffusionPipeline
+from ...image_processor import PipelineImageInput
+from ...loaders import IPAdapterMixin
 from ...schedulers import KarrasDiffusionSchedulers
 from ...utils import PIL_INTERPOLATION, logging
 from ..fastdeploy_utils import FastDeployDiffusionPipelineMixin, FastDeployRuntimeModel
+from ..pipeline_utils import DiffusionPipeline
 from . import StableDiffusionPipelineOutput
 
 logger = logging.get_logger(__name__)  # pylint: disable=invalid-name
 
 
-def prepare_mask_and_masked_image(image,
-                                  mask,
-                                  height=None,
-                                  width=None,
-                                  return_image: bool=False):
+def prepare_mask_and_masked_image(image, mask, height=None, width=None, return_image: bool = False):
     """
     Prepares a pair (image, mask) to be consumed by the Stable Diffusion pipeline. This means that those inputs will be
     converted to ``paddle.Tensor`` with shapes ``batch x channels x height x width`` where ``channels`` is ``3`` for the
@@ -71,14 +69,11 @@ def prepare_mask_and_masked_image(image,
 
     if isinstance(image, paddle.Tensor):
         if not isinstance(mask, paddle.Tensor):
-            raise TypeError(
-                f"`image` is a paddle.Tensor but `mask` (type: {type(mask)} is not"
-            )
+            raise TypeError(f"`image` is a paddle.Tensor but `mask` (type: {type(mask)} is not")
 
         # Batch single image
         if image.ndim == 3:
-            assert image.shape[
-                0] == 3, "Image outside a batch should be of shape (3, H, W)"
+            assert image.shape[0] == 3, "Image outside a batch should be of shape (3, H, W)"
             image = image.unsqueeze(0)
 
         # Batch and add channel dim for single mask
@@ -96,10 +91,8 @@ def prepare_mask_and_masked_image(image,
                 mask = mask.unsqueeze(1)
 
         assert image.ndim == 4 and mask.ndim == 4, "Image and Mask must have 4 dimensions"
-        assert image.shape[-2:] == mask.shape[
-            -2:], "Image and Mask must have the same spatial dimensions"
-        assert image.shape[0] == mask.shape[
-            0], "Image and Mask must have the same batch size"
+        assert image.shape[-2:] == mask.shape[-2:], "Image and Mask must have the same spatial dimensions"
+        assert image.shape[0] == mask.shape[0], "Image and Mask must have the same batch size"
 
         # Check image is in [-1, 1]
         if image.min() < -1 or image.max() > 1:
@@ -116,8 +109,7 @@ def prepare_mask_and_masked_image(image,
         # Image as float32
         image = image.cast(dtype=paddle.float32)
     elif isinstance(mask, paddle.Tensor):
-        raise TypeError(
-            f"`mask` is a paddle.Tensor but `image` (type: {type(image)} is not")
+        raise TypeError(f"`mask` is a paddle.Tensor but `image` (type: {type(image)} is not")
     else:
         # preprocess image
         if isinstance(image, (PIL.Image.Image, np.ndarray)):
@@ -128,13 +120,8 @@ def prepare_mask_and_masked_image(image,
                 w, h = image[0].size
             else:
                 w, h = width, height
-            w, h = (x - x % 8
-                    for x in (w, h))  # resize to integer multiple of 8
-            image = [
-                i.resize(
-                    (w, h), resample=PIL_INTERPOLATION["lanczos"])
-                for i in image
-            ]
+            w, h = (x - x % 8 for x in (w, h))  # resize to integer multiple of 8
+            image = [i.resize((w, h), resample=PIL_INTERPOLATION["lanczos"]) for i in image]
             image = [np.array(i.convert("RGB"))[None, :] for i in image]
             image = np.concatenate(image, axis=0)
         elif isinstance(image, list) and isinstance(image[0], np.ndarray):
@@ -153,14 +140,9 @@ def prepare_mask_and_masked_image(image,
                 w, h = mask[0].size
             else:
                 w, h = width, height
-            w, h = (x - x % 8
-                    for x in (w, h))  # resize to integer multiple of 8
-            mask = [
-                i.resize(
-                    (w, h), resample=PIL_INTERPOLATION["lanczos"]) for i in mask
-            ]
-            mask = np.concatenate(
-                [np.array(m.convert("L"))[None, None, :] for m in mask], axis=0)
+            w, h = (x - x % 8 for x in (w, h))  # resize to integer multiple of 8
+            mask = [i.resize((w, h), resample=PIL_INTERPOLATION["lanczos"]) for i in mask]
+            mask = np.concatenate([np.array(m.convert("L"))[None, None, :] for m in mask], axis=0)
             mask = mask.astype(np.float32) / 255.0
         elif isinstance(mask, list) and isinstance(mask[0], np.ndarray):
             mask = np.concatenate([m[None, None, :] for m in mask], axis=0)
@@ -178,8 +160,7 @@ def prepare_mask_and_masked_image(image,
     return mask, masked_image
 
 
-class FastDeployStableDiffusionInpaintPipeline(
-        DiffusionPipeline, FastDeployDiffusionPipelineMixin):
+class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline, FastDeployDiffusionPipelineMixin, IPAdapterMixin):
     r"""
     Pipeline for text-guided image inpainting using Stable Diffusion.
 
@@ -209,19 +190,21 @@ class FastDeployStableDiffusionInpaintPipeline(
         feature_extractor ([`CLIPImageProcessor`]):
             Model that extracts features from generated images to be used as inputs for the `safety_checker`.
     """
-    _optional_components = ["safety_checker", "feature_extractor"]
+    _optional_components = ["image_encoder", "safety_checker", "feature_extractor"]
 
     def __init__(
-            self,
-            vae_encoder: FastDeployRuntimeModel,
-            vae_decoder: FastDeployRuntimeModel,
-            text_encoder: FastDeployRuntimeModel,
-            tokenizer: CLIPTokenizer,
-            unet: FastDeployRuntimeModel,
-            scheduler: KarrasDiffusionSchedulers,
-            safety_checker: FastDeployRuntimeModel,
-            feature_extractor: CLIPImageProcessor,
-            requires_safety_checker: bool=False, ):
+        self,
+        vae_encoder: FastDeployRuntimeModel,
+        vae_decoder: FastDeployRuntimeModel,
+        text_encoder: FastDeployRuntimeModel,
+        tokenizer: CLIPTokenizer,
+        unet: FastDeployRuntimeModel,
+        scheduler: KarrasDiffusionSchedulers,
+        safety_checker: FastDeployRuntimeModel,
+        feature_extractor: CLIPImageProcessor,
+        image_encoder: FastDeployRuntimeModel,
+        requires_safety_checker: bool = False,
+    ):
         super().__init__()
         if safety_checker is None and requires_safety_checker:
             logger.warning(
@@ -246,38 +229,43 @@ class FastDeployStableDiffusionInpaintPipeline(
             unet=unet,
             scheduler=scheduler,
             safety_checker=safety_checker,
-            feature_extractor=feature_extractor, )
+            feature_extractor=feature_extractor,
+            image_encoder=image_encoder,
+        )
         self.register_to_config(requires_safety_checker=requires_safety_checker)
         self.post_init()
 
     def __call__(
-            self,
-            prompt: Union[str, List[str]]=None,
-            image: Union[paddle.Tensor, PIL.Image.Image]=None,
-            mask_image: Union[paddle.Tensor, PIL.Image.Image]=None,
-            height: int=None,
-            width: int=None,
-            strength: float=1.0,
-            num_inference_steps: int=50,
-            guidance_scale: float=7.5,
-            negative_prompt: Optional[Union[str, List[str]]]=None,
-            num_images_per_prompt: Optional[int]=1,
-            add_predicted_noise: Optional[bool]=False,
-            eta: float=0.0,
-            generator: Optional[Union[paddle.Generator, List[
-                paddle.Generator]]]=None,
-            latents: Optional[paddle.Tensor]=None,
-            parse_prompt_type: Optional[str]="lpw",
-            max_embeddings_multiples: Optional[int]=3,
-            prompt_embeds: Optional[paddle.Tensor]=None,
-            negative_prompt_embeds: Optional[paddle.Tensor]=None,
-            output_type: Optional[str]="pil",
-            return_dict: bool=True,
-            callback: Optional[Callable[[int, int, paddle.Tensor], None]]=None,
-            callback_steps: Optional[int]=1,
-            controlnet_cond: Union[paddle.Tensor, PIL.Image.Image]=None,
-            controlnet_conditioning_scale: float=1.0,
-            infer_op_dict: Dict[str, str]=None, ):
+        self,
+        prompt: Union[str, List[str]] = None,
+        image: Union[paddle.Tensor, PIL.Image.Image] = None,
+        mask_image: Union[paddle.Tensor, PIL.Image.Image] = None,
+        height: int = None,
+        width: int = None,
+        strength: float = 1.0,
+        num_inference_steps: int = 50,
+        guidance_scale: float = 7.5,
+        negative_prompt: Optional[Union[str, List[str]]] = None,
+        num_images_per_prompt: Optional[int] = 1,
+        timesteps: List[int] = None,
+        add_predicted_noise: Optional[bool] = False,
+        eta: float = 0.0,
+        generator: Optional[Union[paddle.Generator, List[paddle.Generator]]] = None,
+        latents: Optional[paddle.Tensor] = None,
+        parse_prompt_type: Optional[str] = "lpw",
+        max_embeddings_multiples: Optional[int] = 3,
+        prompt_embeds: Optional[paddle.Tensor] = None,
+        negative_prompt_embeds: Optional[paddle.Tensor] = None,
+        guidance_rescale: float = 0.0,
+        ip_adapter_image: Optional[PipelineImageInput] = None,
+        output_type: Optional[str] = "pil",
+        return_dict: bool = True,
+        callback: Optional[Callable[[int, int, paddle.Tensor], None]] = None,
+        callback_steps: Optional[int] = 1,
+        controlnet_cond: Union[paddle.Tensor, PIL.Image.Image] = None,
+        controlnet_conditioning_scale: float = 1.0,
+        infer_op_dict: Dict[str, str] = None,
+    ):
         r"""
         Function invoked when calling the pipeline for generation.
 
@@ -336,6 +324,7 @@ class FastDeployStableDiffusionInpaintPipeline(
                 Pre-generated negative text embeddings. Can be used to easily tweak text inputs, *e.g.* prompt
                 weighting. If not provided, negative_prompt_embeds will be generated from `negative_prompt` input
                 argument.
+            ip_adapter_image: (`PipelineImageInput`, *optional*): Optional image input to work with IP Adapters.
             output_type (`str`, *optional*, defaults to `"pil"`):
                 The output format of the generate image. Choose between
                 [PIL](https://pillow.readthedocs.io/en/stable/): `PIL.Image.Image` or `np.array`.
@@ -362,7 +351,8 @@ class FastDeployStableDiffusionInpaintPipeline(
             mask_image,
             height,
             width,
-            return_image=True, )
+            return_image=True,
+        )
         height, width = init_image.shape[-2:]
 
         # 1. Check inputs
@@ -374,7 +364,8 @@ class FastDeployStableDiffusionInpaintPipeline(
             negative_prompt,
             prompt_embeds,
             negative_prompt_embeds,
-            strength, )
+            strength,
+        )
         infer_op_dict = self.prepare_infer_op_dict(infer_op_dict)
 
         # 2. Define call parameters
@@ -400,15 +391,21 @@ class FastDeployStableDiffusionInpaintPipeline(
             negative_prompt_embeds=negative_prompt_embeds,
             parse_prompt_type=parse_prompt_type,
             max_embeddings_multiples=max_embeddings_multiples,
-            infer_op=infer_op_dict.get("text_encoder", None), )
+            infer_op=infer_op_dict.get("text_encoder", None),
+        )
+
+        if ip_adapter_image is not None:
+            image_embeds, negative_image_embeds = self.encode_image(
+                ip_adapter_image, num_images_per_prompt, infer_op=infer_op_dict.get("image_encoder", None)
+            )
+            if do_classifier_free_guidance:
+                image_embeds = paddle.concat([negative_image_embeds, image_embeds])
 
         # 4. set timesteps
-        self.scheduler.set_timesteps(num_inference_steps)
-        timesteps, num_inference_steps = self.get_timesteps(num_inference_steps,
-                                                            strength)
+        timesteps, num_inference_steps = self.retrieve_timesteps(self.scheduler, num_inference_steps, timesteps)
+        timesteps, num_inference_steps = self.get_timesteps(num_inference_steps, strength)
         # at which timestep to set the initial noise (n.b. 50% if strength is 0.5)
-        latent_timestep = timesteps[:1].tile(
-            [batch_size * num_images_per_prompt])
+        latent_timestep = timesteps[:1].tile([batch_size * num_images_per_prompt])
         # create a boolean to check if the strength is set to 1. if so then initialise the latents with pure noise
         is_strength_max = strength == 1.0
 
@@ -428,7 +425,8 @@ class FastDeployStableDiffusionInpaintPipeline(
             is_strength_max=is_strength_max,
             return_noise=True,
             return_image_latents=return_image_latents,
-            infer_op=infer_op_dict.get("vae_encoder", None), )
+            infer_op=infer_op_dict.get("vae_encoder", None),
+        )
 
         if return_image_latents:
             latents, noise, image_latents = latents_outputs
@@ -444,7 +442,8 @@ class FastDeployStableDiffusionInpaintPipeline(
             width,
             do_classifier_free_guidance,
             return_masked_image_latents=True,
-            infer_op=infer_op_dict.get("vae_encoder", None), )
+            infer_op=infer_op_dict.get("vae_encoder", None),
+        )
 
         # 7. Check that sizes of mask, masked image and latents match
         if num_channels_unet == 9:
@@ -456,11 +455,10 @@ class FastDeployStableDiffusionInpaintPipeline(
                     f"Incorrect configuration settings! Received `num_channels_latents`: {num_channels_latents} +"
                     f" `num_channels_mask`: {num_channels_mask} + `num_channels_masked_image`: {num_channels_masked_image}"
                     f" = {num_channels_latents+num_channels_masked_image+num_channels_mask}. Please verify the config of"
-                    " `pipeline.unet` or your `mask_image` or `image` input.")
+                    " `pipeline.unet` or your `mask_image` or `image` input."
+                )
         elif num_channels_unet != 4:
-            raise ValueError(
-                f"The unet should have either 4 or 9 input channels, not {num_channels_unet}."
-            )
+            raise ValueError(f"The unet should have either 4 or 9 input channels, not {num_channels_unet}.")
         # do_controlnet
         do_controlnet = controlnet_cond is not None and num_channels_unet == 4
         if do_controlnet:
@@ -471,107 +469,101 @@ class FastDeployStableDiffusionInpaintPipeline(
                 height=height,
                 batch_size=batch_size,
                 num_images_per_prompt=num_images_per_prompt,
-                do_classifier_free_guidance=do_classifier_free_guidance, )
+                do_classifier_free_guidance=do_classifier_free_guidance,
+            )
 
         # 8. Prepare extra step kwargs. TODO: Logic should ideally just be moved out of the pipeline
         extra_step_kwargs = self.prepare_extra_step_kwargs(generator, eta)
 
         if do_classifier_free_guidance:
-            init_mask = mask[:mask.shape[0] // 2]
+            init_mask = mask[: mask.shape[0] // 2]
         else:
             init_mask = mask
 
         # 9. Denoising loop
-        num_warmup_steps = len(
-            timesteps) - num_inference_steps * self.scheduler.order
+        num_warmup_steps = len(timesteps) - num_inference_steps * self.scheduler.order
         is_scheduler_support_step_index = self.is_scheduler_support_step_index()
 
         with self.progress_bar(total=num_inference_steps) as progress_bar:
             for i, t in enumerate(timesteps):
                 # expand the latents if we are doing classifier free guidance
-                latent_model_input = paddle.concat(
-                    [latents] * 2) if do_classifier_free_guidance else latents
+                latent_model_input = paddle.concat([latents] * 2) if do_classifier_free_guidance else latents
                 if is_scheduler_support_step_index:
-                    latent_model_input = self.scheduler.scale_model_input(
-                        latent_model_input, t, step_index=i)
+                    latent_model_input = self.scheduler.scale_model_input(latent_model_input, t, step_index=i)
                 else:
-                    latent_model_input = self.scheduler.scale_model_input(
-                        latent_model_input, t)
+                    latent_model_input = self.scheduler.scale_model_input(latent_model_input, t)
 
                 output_shape = latent_model_input.shape
                 if not is_legacy:
                     # concat latents, mask, masked_image_latents in the channel dimension
-                    latent_model_input = paddle.concat(
-                        [latent_model_input, mask, masked_image_latents],
-                        axis=1)
+                    latent_model_input = paddle.concat([latent_model_input, mask, masked_image_latents], axis=1)
 
                 unet_inputs = dict(
                     sample=latent_model_input,
                     timestep=t,
                     encoder_hidden_states=prompt_embeds,
                     infer_op=infer_op_dict.get("unet", None),
-                    output_shape=output_shape, )
+                    output_shape=output_shape,
+                )
                 if do_controlnet:
                     unet_inputs["controlnet_cond"] = control_image
-                    unet_inputs[
-                        "controlnet_conditioning_scale"] = control_conditioning_scale
+                    unet_inputs["controlnet_conditioning_scale"] = control_conditioning_scale
+                # Add image embeds for IP-Adapter
+                # added_cond_kwargs = {"image_embeds": image_embeds} if ip_adapter_image is not None else None
+                if ip_adapter_image:
+                    unet_inputs["image_embeds"] = image_embeds
+
                 # predict the noise residual
                 noise_pred_unet = self.unet(**unet_inputs)[0]
 
                 # perform guidance
                 if do_classifier_free_guidance:
-                    noise_pred_uncond, noise_pred_text = noise_pred_unet.chunk(
-                        2)
-                    noise_pred = noise_pred_uncond + guidance_scale * (
-                        noise_pred_text - noise_pred_uncond)
+                    noise_pred_uncond, noise_pred_text = noise_pred_unet.chunk(2)
+                    noise_pred = noise_pred_uncond + guidance_scale * (noise_pred_text - noise_pred_uncond)
+
+                    if guidance_rescale > 0.0:
+                        # Based on 3.4. in https://arxiv.org/pdf/2305.08891.pdf
+                        noise_pred = self.rescale_noise_cfg(
+                            noise_pred, noise_pred_text, guidance_rescale=guidance_rescale
+                        )
                 else:
                     noise_pred = noise_pred_unet
 
                 # compute the previous noisy sample x_t -> x_t-1
                 if is_scheduler_support_step_index:
                     scheduler_output = self.scheduler.step(
-                        noise_pred,
-                        t,
-                        latents,
-                        step_index=i,
-                        return_pred_original_sample=False,
-                        **extra_step_kwargs)
+                        noise_pred, t, latents, step_index=i, return_pred_original_sample=False, **extra_step_kwargs
+                    )
                 else:
-                    scheduler_output = self.scheduler.step(
-                        noise_pred, t, latents, **extra_step_kwargs)
+                    scheduler_output = self.scheduler.step(noise_pred, t, latents, **extra_step_kwargs)
                 latents = scheduler_output.prev_sample
 
                 if is_legacy:
                     if i < len(timesteps) - 1:
                         # masking
                         if add_predicted_noise:
-                            init_latents_proper = self.scheduler.add_noise(
-                                image_latents, noise_pred_uncond, t)
+                            init_latents_proper = self.scheduler.add_noise(image_latents, noise_pred_uncond, t)
                         else:
                             # https://github.com/huggingface/diffusers/pull/3749/files#diff-39d36ab1e622684e35fe6971c12fb44e24756bdc383aba3d7f6e3b1625bdaafc
                             noise_timestep = timesteps[i + 1]
-                            init_latents_proper = self.scheduler.add_noise(
-                                image_latents, noise, noise_timestep)
+                            init_latents_proper = self.scheduler.add_noise(image_latents, noise, noise_timestep)
                     else:
                         init_latents_proper = image_latents
-                    latents = (1 - init_mask
-                               ) * init_latents_proper + init_mask * latents
+                    latents = (1 - init_mask) * init_latents_proper + init_mask * latents
 
                 # call the callback, if provided
-                if i == len(timesteps) - 1 or (
-                    (i + 1) > num_warmup_steps and
-                    (i + 1) % self.scheduler.order == 0):
+                if i == len(timesteps) - 1 or ((i + 1) > num_warmup_steps and (i + 1) % self.scheduler.order == 0):
                     progress_bar.update()
                     if callback is not None and i % callback_steps == 0:
                         callback(i, t, latents)
                     if i == len(timesteps) - 1:
                         # sync for accuracy it/s measure
-                        paddle.device.cuda.synchronize()
+                        paddle.device.synchronize()
 
         if not output_type == "latent":
             image = self._decode_vae_latents(
-                latents / self.vae_scaling_factor,
-                infer_op=infer_op_dict.get("vae_decoder", None))
+                latents / self.vae_scaling_factor, infer_op=infer_op_dict.get("vae_decoder", None)
+            )
             image, has_nsfw_concept = self.run_safety_checker(image)
         else:
             image = latents
@@ -582,11 +574,9 @@ class FastDeployStableDiffusionInpaintPipeline(
         else:
             do_denormalize = [not has_nsfw for has_nsfw in has_nsfw_concept]
 
-        image = self.image_processor.postprocess(
-            image, output_type=output_type, do_denormalize=do_denormalize)
+        image = self.image_processor.postprocess(image, output_type=output_type, do_denormalize=do_denormalize)
 
         if not return_dict:
             return (image, has_nsfw_concept)
 
-        return StableDiffusionPipelineOutput(
-            images=image, nsfw_content_detected=has_nsfw_concept)
+        return StableDiffusionPipelineOutput(images=image, nsfw_content_detected=has_nsfw_concept)
