@@ -1,4 +1,4 @@
-# Copyright 2022 Kakao Brain and The HuggingFace Team. All rights reserved.
+# Copyright 2023 Kakao Brain and The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,16 @@ import inspect
 from typing import List, Optional, Tuple, Union
 
 import paddle
-import paddle.nn.functional as F
+from paddle.nn import functional as F
 
-from paddlenlp.transformers import CLIPTextModelWithProjection, CLIPTokenizer
-from paddlenlp.transformers.clip.modeling import CLIPTextModelOutput
+from ppdiffusers.transformers import CLIPTextModelWithProjection, CLIPTokenizer
+from ppdiffusers.transformers.clip.modeling import CLIPTextModelOutput
 
 from ...models import PriorTransformer, UNet2DConditionModel, UNet2DModel
-from ...pipelines import DiffusionPipeline
-from ...pipelines.pipeline_utils import ImagePipelineOutput
 from ...schedulers import UnCLIPScheduler
-from ...utils import logging, randn_tensor
+from ...utils import logging
+from ...utils.paddle_utils import randn_tensor
+from ..pipeline_utils import DiffusionPipeline, ImagePipelineOutput
 from .text_proj import UnCLIPTextProjModel
 
 logger = logging.get_logger(__name__)  # pylint: disable=invalid-name
@@ -33,36 +33,37 @@ logger = logging.get_logger(__name__)  # pylint: disable=invalid-name
 
 class UnCLIPPipeline(DiffusionPipeline):
     """
-    Pipeline for text-to-image generation using unCLIP
+    Pipeline for text-to-image generation using unCLIP.
 
-    This model inherits from [`DiffusionPipeline`]. Check the superclass documentation for the generic methods the
-    library implements for all the pipelines (such as downloading or saving, running on a particular device, etc.)
+    This model inherits from [`DiffusionPipeline`]. Check the superclass documentation for the generic methods
+    implemented for all pipelines (downloading, saving, running on a particular device, etc.).
 
     Args:
-        text_encoder ([`CLIPTextModelWithProjection`]):
+        text_encoder ([`~transformers.CLIPTextModelWithProjection`]):
             Frozen text-encoder.
-        tokenizer (`CLIPTokenizer`):
-            Tokenizer of class
-            [CLIPTokenizer](https://huggingface.co/docs/transformers/v4.21.0/en/model_doc/clip#transformers.CLIPTokenizer).
+        tokenizer ([`~transformers.CLIPTokenizer`]):
+            A `CLIPTokenizer` to tokenize text.
         prior ([`PriorTransformer`]):
-            The canonincal unCLIP prior to approximate the image embedding from the text embedding.
+            The canonical unCLIP prior to approximate the image embedding from the text embedding.
         text_proj ([`UnCLIPTextProjModel`]):
             Utility class to prepare and combine the embeddings before they are passed to the decoder.
         decoder ([`UNet2DConditionModel`]):
             The decoder to invert the image embedding into an image.
         super_res_first ([`UNet2DModel`]):
-            Super resolution unet. Used in all but the last step of the super resolution diffusion process.
+            Super resolution UNet. Used in all but the last step of the super resolution diffusion process.
         super_res_last ([`UNet2DModel`]):
-            Super resolution unet. Used in the last step of the super resolution diffusion process.
+            Super resolution UNet. Used in the last step of the super resolution diffusion process.
         prior_scheduler ([`UnCLIPScheduler`]):
-            Scheduler used in the prior denoising process. Just a modified DDPMScheduler.
+            Scheduler used in the prior denoising process (a modified [`DDPMScheduler`]).
         decoder_scheduler ([`UnCLIPScheduler`]):
-            Scheduler used in the decoder denoising process. Just a modified DDPMScheduler.
+            Scheduler used in the decoder denoising process (a modified [`DDPMScheduler`]).
         super_res_scheduler ([`UnCLIPScheduler`]):
-            Scheduler used in the super resolution denoising process. Just a modified DDPMScheduler.
+            Scheduler used in the super resolution denoising process (a modified [`DDPMScheduler`]).
 
     """
 
+    _exclude_from_cpu_offload = ["prior"]
+
     prior: PriorTransformer
     decoder: UNet2DConditionModel
     text_proj: UnCLIPTextProjModel
@@ -75,18 +76,21 @@ class UnCLIPPipeline(DiffusionPipeline):
     decoder_scheduler: UnCLIPScheduler
     super_res_scheduler: UnCLIPScheduler
 
+    model_cpu_offload_seq = "text_encoder->text_proj->decoder->super_res_first->super_res_last"
+
     def __init__(
-            self,
-            prior: PriorTransformer,
-            decoder: UNet2DConditionModel,
-            text_encoder: CLIPTextModelWithProjection,
-            tokenizer: CLIPTokenizer,
-            text_proj: UnCLIPTextProjModel,
-            super_res_first: UNet2DModel,
-            super_res_last: UNet2DModel,
-            prior_scheduler: UnCLIPScheduler,
-            decoder_scheduler: UnCLIPScheduler,
-            super_res_scheduler: UnCLIPScheduler, ):
+        self,
+        prior: PriorTransformer,
+        decoder: UNet2DConditionModel,
+        text_encoder: CLIPTextModelWithProjection,
+        tokenizer: CLIPTokenizer,
+        text_proj: UnCLIPTextProjModel,
+        super_res_first: UNet2DModel,
+        super_res_last: UNet2DModel,
+        prior_scheduler: UnCLIPScheduler,
+        decoder_scheduler: UnCLIPScheduler,
+        super_res_scheduler: UnCLIPScheduler,
+    ):
         super().__init__()
 
         self.register_modules(
@@ -99,27 +103,28 @@ class UnCLIPPipeline(DiffusionPipeline):
             super_res_last=super_res_last,
             prior_scheduler=prior_scheduler,
             decoder_scheduler=decoder_scheduler,
-            super_res_scheduler=super_res_scheduler, )
+            super_res_scheduler=super_res_scheduler,
+        )
 
     def prepare_latents(self, shape, dtype, generator, latents, scheduler):
         if latents is None:
             latents = randn_tensor(shape, generator=generator, dtype=dtype)
         else:
             if latents.shape != list(shape):
-                raise ValueError(
-                    f"Unexpected latents shape, got {latents.shape}, expected {shape}"
-                )
+                raise ValueError(f"Unexpected latents shape, got {latents.shape}, expected {list(shape)}")
+            latents = latents.cast(dtype)
 
         latents = latents * scheduler.init_noise_sigma
         return latents
 
     def _encode_prompt(
-            self,
-            prompt,
-            num_images_per_prompt,
-            do_classifier_free_guidance,
-            text_model_output: Optional[Union[CLIPTextModelOutput, Tuple]]=None,
-            text_attention_mask: Optional[paddle.Tensor]=None, ):
+        self,
+        prompt,
+        num_images_per_prompt,
+        do_classifier_free_guidance,
+        text_model_output: Optional[Union[CLIPTextModelOutput, Tuple]] = None,
+        text_attention_mask: Optional[paddle.Tensor] = None,
+    ):
         if text_model_output is None:
             batch_size = len(prompt) if isinstance(prompt, list) else 1
             # get prompt text embeddings
@@ -127,59 +132,40 @@ class UnCLIPPipeline(DiffusionPipeline):
                 prompt,
                 padding="max_length",
                 max_length=self.tokenizer.model_max_length,
-                truncation=True,
                 return_attention_mask=True,
-                return_tensors="pd", )
+                truncation=True,
+                return_tensors="pd",
+            )
             text_input_ids = text_inputs.input_ids
             text_mask = text_inputs.attention_mask
 
-            untruncated_ids = self.tokenizer(
-                prompt, padding="longest", return_tensors="pd").input_ids
+            untruncated_ids = self.tokenizer(prompt, padding="longest", return_tensors="pd").input_ids
 
-            if untruncated_ids.shape[-1] >= text_input_ids.shape[
-                    -1] and not paddle.equal_all(text_input_ids,
-                                                 untruncated_ids):
+            if untruncated_ids.shape[-1] >= text_input_ids.shape[-1] and not paddle.equal_all(
+                text_input_ids, untruncated_ids
+            ):
                 removed_text = self.tokenizer.batch_decode(
-                    untruncated_ids[:, self.tokenizer.model_max_length - 1:-1])
+                    untruncated_ids[:, self.tokenizer.model_max_length - 1 : -1]
+                )
                 logger.warning(
                     "The following part of your input was truncated because CLIP can only handle sequences up to"
-                    f" {self.tokenizer.model_max_length} tokens: {removed_text}")
-                text_input_ids = text_input_ids[:, :
-                                                self.tokenizer.model_max_length]
+                    f" {self.tokenizer.model_max_length} tokens: {removed_text}"
+                )
+                text_input_ids = text_input_ids[:, : self.tokenizer.model_max_length]
 
             text_encoder_output = self.text_encoder(text_input_ids)
 
             prompt_embeds = text_encoder_output.text_embeds
-            text_encoder_hidden_states = text_encoder_output.last_hidden_state
+            text_enc_hid_states = text_encoder_output.last_hidden_state
 
         else:
             batch_size = text_model_output[0].shape[0]
-            prompt_embeds, text_encoder_hidden_states = text_model_output[
-                0], text_model_output[1]
+            prompt_embeds, text_enc_hid_states = text_model_output[0], text_model_output[1]
             text_mask = text_attention_mask
 
-        # duplicate text embeddings for each generation per prompt
-        seq_len = prompt_embeds.shape[1]
-        prompt_embeds = prompt_embeds.tile([1, num_images_per_prompt])
-        prompt_embeds = prompt_embeds.reshape(
-            [batch_size * num_images_per_prompt, seq_len])
-
-        # duplicate text_encoder_hidden_states for each generation per prompt
-        seq_len = text_encoder_hidden_states.shape[1]
-        text_encoder_hidden_states = text_encoder_hidden_states.tile(
-            [1, num_images_per_prompt, 1])
-        text_encoder_hidden_states = text_encoder_hidden_states.reshape(
-            [batch_size * num_images_per_prompt, seq_len, -1])
-
-        # duplicate text_mask for each generation per prompt
-        seq_len = text_mask.shape[1]
-        text_mask = text_mask.tile([1, num_images_per_prompt])
-        text_mask = text_mask.reshape(
-            [batch_size * num_images_per_prompt, seq_len])
-
-        # prompt_embeds = prompt_embeds.repeat_interleave(num_images_per_prompt, axis=0)
-        # text_encoder_hidden_states = text_encoder_hidden_states.repeat_interleave(num_images_per_prompt, axis=0)
-        # text_mask = text_mask.repeat_interleave(num_images_per_prompt, axis=0)
+        prompt_embeds = prompt_embeds.repeat_interleave(num_images_per_prompt, axis=0)
+        text_enc_hid_states = text_enc_hid_states.repeat_interleave(num_images_per_prompt, axis=0)
+        text_mask = text_mask.repeat_interleave(num_images_per_prompt, axis=0)
 
         if do_classifier_free_guidance:
             uncond_tokens = [""] * batch_size
@@ -190,75 +176,65 @@ class UnCLIPPipeline(DiffusionPipeline):
                 max_length=self.tokenizer.model_max_length,
                 return_attention_mask=True,
                 truncation=True,
-                return_tensors="pd", )
+                return_tensors="pd",
+            )
             uncond_text_mask = uncond_input.attention_mask
-            negative_prompt_embeds_text_encoder_output = self.text_encoder(
-                uncond_input.input_ids)
+            negative_prompt_embeds_text_encoder_output = self.text_encoder(uncond_input.input_ids)
 
             negative_prompt_embeds = negative_prompt_embeds_text_encoder_output.text_embeds
-            uncond_text_encoder_hidden_states = negative_prompt_embeds_text_encoder_output.last_hidden_state
+            uncond_text_enc_hid_states = negative_prompt_embeds_text_encoder_output.last_hidden_state
 
             # duplicate unconditional embeddings for each generation per prompt, using mps friendly method
 
             seq_len = negative_prompt_embeds.shape[1]
-            negative_prompt_embeds = negative_prompt_embeds.tile(
-                [1, num_images_per_prompt])
-            negative_prompt_embeds = negative_prompt_embeds.reshape(
-                [batch_size * num_images_per_prompt, seq_len])
+            negative_prompt_embeds = negative_prompt_embeds.tile([1, num_images_per_prompt])
+            negative_prompt_embeds = negative_prompt_embeds.reshape([batch_size * num_images_per_prompt, seq_len])
 
-            seq_len = uncond_text_encoder_hidden_states.shape[1]
-            uncond_text_encoder_hidden_states = uncond_text_encoder_hidden_states.tile(
-                [1, num_images_per_prompt, 1])
-            uncond_text_encoder_hidden_states = uncond_text_encoder_hidden_states.reshape(
-                [batch_size * num_images_per_prompt, seq_len, -1])
+            seq_len = uncond_text_enc_hid_states.shape[1]
+            uncond_text_enc_hid_states = uncond_text_enc_hid_states.tile([1, num_images_per_prompt, 1])
+            uncond_text_enc_hid_states = uncond_text_enc_hid_states.reshape(
+                [batch_size * num_images_per_prompt, seq_len, -1]
+            )
+            uncond_text_mask = uncond_text_mask.repeat_interleave(num_images_per_prompt, axis=0)
 
-            # duplicate uncond_text_mask for each generation per prompt
-            seq_len = uncond_text_mask.shape[1]
-            uncond_text_mask = uncond_text_mask.tile([1, num_images_per_prompt])
-            uncond_text_mask = uncond_text_mask.reshape(
-                [batch_size * num_images_per_prompt, seq_len])
-            # uncond_text_mask = uncond_text_mask.repeat_interleave(num_images_per_prompt, axis=0)
             # done duplicates
 
             # For classifier free guidance, we need to do two forward passes.
             # Here we concatenate the unconditional and text embeddings into a single batch
             # to avoid doing two forward passes
-            prompt_embeds = paddle.concat(
-                [negative_prompt_embeds, prompt_embeds])
-            text_encoder_hidden_states = paddle.concat([
-                uncond_text_encoder_hidden_states, text_encoder_hidden_states
-            ])
+            prompt_embeds = paddle.concat([negative_prompt_embeds, prompt_embeds])
+            text_enc_hid_states = paddle.concat([uncond_text_enc_hid_states, text_enc_hid_states])
 
             text_mask = paddle.concat([uncond_text_mask, text_mask])
 
-        return prompt_embeds, text_encoder_hidden_states, text_mask
+        return prompt_embeds, text_enc_hid_states, text_mask
 
     @paddle.no_grad()
     def __call__(
-            self,
-            prompt: Optional[Union[str, List[str]]]=None,
-            num_images_per_prompt: int=1,
-            prior_num_inference_steps: int=25,
-            decoder_num_inference_steps: int=25,
-            super_res_num_inference_steps: int=7,
-            generator: Optional[Union[paddle.Generator, List[
-                paddle.Generator]]]=None,
-            prior_latents: Optional[paddle.Tensor]=None,
-            decoder_latents: Optional[paddle.Tensor]=None,
-            super_res_latents: Optional[paddle.Tensor]=None,
-            text_model_output: Optional[Union[CLIPTextModelOutput, Tuple]]=None,
-            text_attention_mask: Optional[paddle.Tensor]=None,
-            prior_guidance_scale: float=4.0,
-            decoder_guidance_scale: float=8.0,
-            output_type: Optional[str]="pil",
-            return_dict: bool=True, ):
+        self,
+        prompt: Optional[Union[str, List[str]]] = None,
+        num_images_per_prompt: int = 1,
+        prior_num_inference_steps: int = 25,
+        decoder_num_inference_steps: int = 25,
+        super_res_num_inference_steps: int = 7,
+        generator: Optional[Union[paddle.Generator, List[paddle.Generator]]] = None,
+        prior_latents: Optional[paddle.Tensor] = None,
+        decoder_latents: Optional[paddle.Tensor] = None,
+        super_res_latents: Optional[paddle.Tensor] = None,
+        text_model_output: Optional[Union[CLIPTextModelOutput, Tuple]] = None,
+        text_attention_mask: Optional[paddle.Tensor] = None,
+        prior_guidance_scale: float = 4.0,
+        decoder_guidance_scale: float = 8.0,
+        output_type: Optional[str] = "pil",
+        return_dict: bool = True,
+    ):
         """
-        Function invoked when calling the pipeline for generation.
+        The call function to the pipeline for generation.
 
         Args:
             prompt (`str` or `List[str]`):
-                The prompt or prompts to guide the image generation. This can only be left undefined if
-                `text_model_output` and `text_attention_mask` is passed.
+                The prompt or prompts to guide image generation. This can only be left undefined if `text_model_output`
+                and `text_attention_mask` is passed.
             num_images_per_prompt (`int`, *optional*, defaults to 1):
                 The number of images to generate per prompt.
             prior_num_inference_steps (`int`, *optional*, defaults to 25):
@@ -271,7 +247,7 @@ class UnCLIPPipeline(DiffusionPipeline):
                 The number of denoising steps for super resolution. More denoising steps usually lead to a higher
                 quality image at the expense of slower inference.
             generator (`paddle.Generator` or `List[paddle.Generator]`, *optional*):
-                One or a list of paddle generator(s) to make generation deterministic.
+                A [`paddle.Generator`] to make generation deterministic.
             prior_latents (`paddle.Tensor` of shape (batch size, embeddings dimension), *optional*):
                 Pre-generated noisy latents to be used as inputs for the prior.
             decoder_latents (`paddle.Tensor` of shape (batch size, channels, height, width), *optional*):
@@ -279,29 +255,27 @@ class UnCLIPPipeline(DiffusionPipeline):
             super_res_latents (`paddle.Tensor` of shape (batch size, channels, super res height, super res width), *optional*):
                 Pre-generated noisy latents to be used as inputs for the decoder.
             prior_guidance_scale (`float`, *optional*, defaults to 4.0):
-                Guidance scale as defined in [Classifier-Free Diffusion Guidance](https://arxiv.org/abs/2207.12598).
-                `guidance_scale` is defined as `w` of equation 2. of [Imagen
-                Paper](https://arxiv.org/pdf/2205.11487.pdf). Guidance scale is enabled by setting `guidance_scale >
-                1`. Higher guidance scale encourages to generate images that are closely linked to the text `prompt`,
-                usually at the expense of lower image quality.
+                A higher guidance scale value encourages the model to generate images closely linked to the text
+                `prompt` at the expense of lower image quality. Guidance scale is enabled when `guidance_scale > 1`.
             decoder_guidance_scale (`float`, *optional*, defaults to 4.0):
-                Guidance scale as defined in [Classifier-Free Diffusion Guidance](https://arxiv.org/abs/2207.12598).
-                `guidance_scale` is defined as `w` of equation 2. of [Imagen
-                Paper](https://arxiv.org/pdf/2205.11487.pdf). Guidance scale is enabled by setting `guidance_scale >
-                1`. Higher guidance scale encourages to generate images that are closely linked to the text `prompt`,
-                usually at the expense of lower image quality.
+                A higher guidance scale value encourages the model to generate images closely linked to the text
+                `prompt` at the expense of lower image quality. Guidance scale is enabled when `guidance_scale > 1`.
             text_model_output (`CLIPTextModelOutput`, *optional*):
-                Pre-defined CLIPTextModel outputs that can be derived from the text encoder. Pre-defined text outputs
-                can be passed for tasks like text embedding interpolations. Make sure to also pass
-                `text_attention_mask` in this case. `prompt` can the be left to `None`.
+                Pre-defined [`CLIPTextModel`] outputs that can be derived from the text encoder. Pre-defined text
+                outputs can be passed for tasks like text embedding interpolations. Make sure to also pass
+                `text_attention_mask` in this case. `prompt` can the be left `None`.
             text_attention_mask (`paddle.Tensor`, *optional*):
                 Pre-defined CLIP text attention mask that can be derived from the tokenizer. Pre-defined text attention
                 masks are necessary when passing `text_model_output`.
             output_type (`str`, *optional*, defaults to `"pil"`):
-                The output format of the generated image. Choose between
-                [PIL](https://pillow.readthedocs.io/en/stable/): `PIL.Image.Image` or `np.array`.
+                The output format of the generated image. Choose between `PIL.Image` or `np.array`.
             return_dict (`bool`, *optional*, defaults to `True`):
                 Whether or not to return a [`~pipelines.ImagePipelineOutput`] instead of a plain tuple.
+
+        Returns:
+            [`~pipelines.ImagePipelineOutput`] or `tuple`:
+                If `return_dict` is `True`, [`~pipelines.ImagePipelineOutput`] is returned, otherwise a `tuple` is
+                returned where the first element is a list with the generated images.
         """
         if prompt is not None:
             if isinstance(prompt, str):
@@ -309,9 +283,7 @@ class UnCLIPPipeline(DiffusionPipeline):
             elif isinstance(prompt, list):
                 batch_size = len(prompt)
             else:
-                raise ValueError(
-                    f"`prompt` has to be of type `str` or `list` but is {type(prompt)}"
-                )
+                raise ValueError(f"`prompt` has to be of type `str` or `list` but is {type(prompt)}")
         else:
             batch_size = text_model_output[0].shape[0]
 
@@ -319,9 +291,9 @@ class UnCLIPPipeline(DiffusionPipeline):
 
         do_classifier_free_guidance = prior_guidance_scale > 1.0 or decoder_guidance_scale > 1.0
 
-        prompt_embeds, text_encoder_hidden_states, text_mask = self._encode_prompt(
-            prompt, num_images_per_prompt, do_classifier_free_guidance,
-            text_model_output, text_attention_mask)
+        prompt_embeds, text_enc_hid_states, text_mask = self._encode_prompt(
+            prompt, num_images_per_prompt, do_classifier_free_guidance, text_model_output, text_attention_mask
+        )
 
         # prior
 
@@ -335,27 +307,26 @@ class UnCLIPPipeline(DiffusionPipeline):
             prompt_embeds.dtype,
             generator,
             prior_latents,
-            self.prior_scheduler, )
+            self.prior_scheduler,
+        )
 
         for i, t in enumerate(self.progress_bar(prior_timesteps_tensor)):
             # expand the latents if we are doing classifier free guidance
-            latent_model_input = paddle.concat(
-                [prior_latents] *
-                2) if do_classifier_free_guidance else prior_latents
+            latent_model_input = paddle.concat([prior_latents] * 2) if do_classifier_free_guidance else prior_latents
 
             predicted_image_embedding = self.prior(
                 latent_model_input,
                 timestep=t,
                 proj_embedding=prompt_embeds,
-                encoder_hidden_states=text_encoder_hidden_states,
-                attention_mask=text_mask, ).predicted_image_embedding
+                encoder_hidden_states=text_enc_hid_states,
+                attention_mask=text_mask,
+            ).predicted_image_embedding
 
             if do_classifier_free_guidance:
-                predicted_image_embedding_uncond, predicted_image_embedding_text = predicted_image_embedding.chunk(
-                    2)
+                predicted_image_embedding_uncond, predicted_image_embedding_text = predicted_image_embedding.chunk(2)
                 predicted_image_embedding = predicted_image_embedding_uncond + prior_guidance_scale * (
-                    predicted_image_embedding_text -
-                    predicted_image_embedding_uncond)
+                    predicted_image_embedding_text - predicted_image_embedding_uncond
+                )
 
             if i + 1 == prior_timesteps_tensor.shape[0]:
                 prev_timestep = None
@@ -367,7 +338,8 @@ class UnCLIPPipeline(DiffusionPipeline):
                 timestep=t,
                 sample=prior_latents,
                 generator=generator,
-                prev_timestep=prev_timestep, ).prev_sample
+                prev_timestep=prev_timestep,
+            ).prev_sample
 
         prior_latents = self.prior.post_process_latents(prior_latents)
 
@@ -376,16 +348,17 @@ class UnCLIPPipeline(DiffusionPipeline):
         # done prior
 
         # decoder
-        text_encoder_hidden_states, additive_clip_time_embeddings = self.text_proj(
+
+        text_enc_hid_states, additive_clip_time_embeddings = self.text_proj(
             image_embeddings=image_embeddings,
             prompt_embeds=prompt_embeds,
-            text_encoder_hidden_states=text_encoder_hidden_states,
-            do_classifier_free_guidance=do_classifier_free_guidance, )
+            text_encoder_hidden_states=text_enc_hid_states,
+            do_classifier_free_guidance=do_classifier_free_guidance,
+        )
 
-        decoder_text_mask = F.pad(text_mask.unsqueeze(0),
-                                  (self.text_proj.clip_extra_context_tokens, 0),
-                                  value=1,
-                                  data_format="NCL").squeeze(0)
+        decoder_text_mask = F.pad(
+            text_mask.unsqueeze(0), (self.text_proj.clip_extra_context_tokens, 0), value=1, data_format="NCL"
+        ).squeeze(0)
 
         self.decoder_scheduler.set_timesteps(decoder_num_inference_steps)
         decoder_timesteps_tensor = self.decoder_scheduler.timesteps
@@ -395,44 +368,37 @@ class UnCLIPPipeline(DiffusionPipeline):
         width = self.decoder.config.sample_size
 
         decoder_latents = self.prepare_latents(
-            (batch_size, num_channels_latents, height, width),
-            text_encoder_hidden_states.dtype,
+            [batch_size, num_channels_latents, height, width],
+            text_enc_hid_states.dtype,
             generator,
             decoder_latents,
-            self.decoder_scheduler, )
+            self.decoder_scheduler,
+        )
 
         for i, t in enumerate(self.progress_bar(decoder_timesteps_tensor)):
             # expand the latents if we are doing classifier free guidance
-            latent_model_input = (paddle.concat([decoder_latents] * 2)
-                                  if do_classifier_free_guidance else
-                                  decoder_latents)
+            latent_model_input = (
+                paddle.concat([decoder_latents] * 2) if do_classifier_free_guidance else decoder_latents
+            )
 
             noise_pred = self.decoder(
                 sample=latent_model_input,
                 timestep=t,
-                encoder_hidden_states=text_encoder_hidden_states,
+                encoder_hidden_states=text_enc_hid_states,
                 class_labels=additive_clip_time_embeddings,
-                attention_mask=decoder_text_mask, ).sample
+                attention_mask=decoder_text_mask,
+            ).sample
 
             if do_classifier_free_guidance:
                 noise_pred_uncond, noise_pred_text = noise_pred.chunk(2)
-                # paddle.split is not equal torch.split
                 noise_pred_uncond, _ = noise_pred_uncond.split(
-                    [
-                        latent_model_input.shape[1],
-                        noise_pred_uncond.shape[1] - latent_model_input.shape[1]
-                    ],
-                    axis=1)
+                    [latent_model_input.shape[1], noise_pred_uncond.shape[1] - latent_model_input.shape[1]], axis=1
+                )
                 noise_pred_text, predicted_variance = noise_pred_text.split(
-                    [
-                        latent_model_input.shape[1],
-                        noise_pred_text.shape[1] - latent_model_input.shape[1]
-                    ],
-                    axis=1)
-                noise_pred = noise_pred_uncond + decoder_guidance_scale * (
-                    noise_pred_text - noise_pred_uncond)
-                noise_pred = paddle.concat(
-                    [noise_pred, predicted_variance], axis=1)
+                    [latent_model_input.shape[1], noise_pred_text.shape[1] - latent_model_input.shape[1]], axis=1
+                )
+                noise_pred = noise_pred_uncond + decoder_guidance_scale * (noise_pred_text - noise_pred_uncond)
+                noise_pred = paddle.concat([noise_pred, predicted_variance], axis=1)
 
             if i + 1 == decoder_timesteps_tensor.shape[0]:
                 prev_timestep = None
@@ -441,11 +407,8 @@ class UnCLIPPipeline(DiffusionPipeline):
 
             # compute the previous noisy sample x_t -> x_t-1
             decoder_latents = self.decoder_scheduler.step(
-                noise_pred,
-                t,
-                decoder_latents,
-                prev_timestep=prev_timestep,
-                generator=generator).prev_sample
+                noise_pred, t, decoder_latents, prev_timestep=prev_timestep, generator=generator
+            ).prev_sample
 
         decoder_latents = decoder_latents.clip(-1, 1)
 
@@ -467,18 +430,16 @@ class UnCLIPPipeline(DiffusionPipeline):
             image_small.dtype,
             generator,
             super_res_latents,
-            self.super_res_scheduler, )
+            self.super_res_scheduler,
+        )
 
         interpolate_antialias = {}
         if "antialias" in inspect.signature(F.interpolate).parameters:
             interpolate_antialias["antialias"] = True
 
         image_upscaled = F.interpolate(
-            image_small,
-            size=[height, width],
-            mode="bicubic",
-            align_corners=False,
-            **interpolate_antialias)
+            image_small, size=[height, width], mode="bicubic", align_corners=False, **interpolate_antialias
+        )
 
         for i, t in enumerate(self.progress_bar(super_res_timesteps_tensor)):
             # no classifier free guidance
@@ -488,16 +449,12 @@ class UnCLIPPipeline(DiffusionPipeline):
             else:
                 unet = self.super_res_first
 
-            latent_model_input = paddle.concat(
-                [
-                    super_res_latents,
-                    image_upscaled.cast(super_res_latents.dtype)
-                ],
-                axis=1)
+            latent_model_input = paddle.concat([super_res_latents, image_upscaled], axis=1)
 
             noise_pred = unet(
                 sample=latent_model_input,
-                timestep=t, ).sample
+                timestep=t,
+            ).sample
 
             if i + 1 == super_res_timesteps_tensor.shape[0]:
                 prev_timestep = None
@@ -506,11 +463,8 @@ class UnCLIPPipeline(DiffusionPipeline):
 
             # compute the previous noisy sample x_t -> x_t-1
             super_res_latents = self.super_res_scheduler.step(
-                noise_pred,
-                t,
-                super_res_latents,
-                prev_timestep=prev_timestep,
-                generator=generator).prev_sample
+                noise_pred, t, super_res_latents, prev_timestep=prev_timestep, generator=generator
+            ).prev_sample
 
         image = super_res_latents
         # done super res
@@ -519,12 +473,12 @@ class UnCLIPPipeline(DiffusionPipeline):
 
         image = image * 0.5 + 0.5
         image = image.clip(0, 1)
-        image = image.transpose([0, 2, 3, 1]).cast("float32").numpy()
+        image = image.transpose([0, 2, 3, 1]).cast("float32").cpu().numpy()
 
         if output_type == "pil":
             image = self.numpy_to_pil(image)
 
         if not return_dict:
-            return (image, )
+            return (image,)
 
         return ImagePipelineOutput(images=image)
