@@ -1,5 +1,4 @@
-# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
-# Copyright 2022 Microsoft and The HuggingFace Team. All rights reserved.
+# Copyright 2023 Microsoft and The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,14 +13,14 @@
 # limitations under the License.
 
 from dataclasses import dataclass
-from typing import List, Optional, Tuple, Union
+from typing import Optional, Tuple, Union
 
 import numpy as np
 import paddle
 import paddle.nn.functional as F
 
 from ..configuration_utils import ConfigMixin, register_to_config
-from ..utils import BaseOutput, rand_tensor
+from ..utils import BaseOutput
 from .scheduling_utils import SchedulerMixin
 
 
@@ -69,45 +68,42 @@ def index_to_log_onehot(x: paddle.Tensor, num_classes: int) -> paddle.Tensor:
     return log_x
 
 
-def gumbel_noised(logits: paddle.Tensor,
-                  generator: Optional[paddle.Generator]) -> paddle.Tensor:
+def gumbel_noised(logits: paddle.Tensor, generator: Optional[paddle.Generator]) -> paddle.Tensor:
     """
     Apply gumbel noise to `logits`
     """
-    uniform = rand_tensor(logits.shape, generator=generator)
+    uniform = paddle.rand(logits.shape, generator=generator)
     gumbel_noise = -paddle.log(-paddle.log(uniform + 1e-30) + 1e-30)
     noised = gumbel_noise + logits
     return noised
 
 
-def alpha_schedules(num_diffusion_timesteps: int,
-                    alpha_cum_start=0.99999,
-                    alpha_cum_end=0.000009):
+def alpha_schedules(num_diffusion_timesteps: int, alpha_cum_start=0.99999, alpha_cum_end=0.000009):
     """
     Cumulative and non-cumulative alpha schedules.
 
     See section 4.1.
     """
-    att = (np.arange(0, num_diffusion_timesteps) /
-           (num_diffusion_timesteps - 1) *
-           (alpha_cum_end - alpha_cum_start) + alpha_cum_start)
+    att = (
+        np.arange(0, num_diffusion_timesteps) / (num_diffusion_timesteps - 1) * (alpha_cum_end - alpha_cum_start)
+        + alpha_cum_start
+    )
     att = np.concatenate(([1], att))
     at = att[1:] / att[:-1]
     att = np.concatenate((att[1:], [1]))
     return at, att
 
 
-def gamma_schedules(num_diffusion_timesteps: int,
-                    gamma_cum_start=0.000009,
-                    gamma_cum_end=0.99999):
+def gamma_schedules(num_diffusion_timesteps: int, gamma_cum_start=0.000009, gamma_cum_end=0.99999):
     """
     Cumulative and non-cumulative gamma schedules.
 
     See section 4.1.
     """
-    ctt = (np.arange(0, num_diffusion_timesteps) /
-           (num_diffusion_timesteps - 1) *
-           (gamma_cum_end - gamma_cum_start) + gamma_cum_start)
+    ctt = (
+        np.arange(0, num_diffusion_timesteps) / (num_diffusion_timesteps - 1) * (gamma_cum_end - gamma_cum_start)
+        + gamma_cum_start
+    )
     ctt = np.concatenate(([0], ctt))
     one_minus_ctt = 1 - ctt
     one_minus_ct = one_minus_ctt[1:] / one_minus_ctt[:-1]
@@ -118,36 +114,24 @@ def gamma_schedules(num_diffusion_timesteps: int,
 
 class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
     """
-    The VQ-diffusion transformer outputs predicted probabilities of the initial unnoised image.
+    A scheduler for vector quantized diffusion.
 
-    The VQ-diffusion scheduler converts the transformer's output into a sample for the unnoised image at the previous
-    diffusion timestep.
-
-    [`~ConfigMixin`] takes care of storing all config attributes that are passed in the scheduler's `__init__`
-    function, such as `num_train_timesteps`. They can be accessed via `scheduler.config.num_train_timesteps`.
-    [`SchedulerMixin`] provides general loading and saving functionality via the [`SchedulerMixin.save_pretrained`] and
-    [`~SchedulerMixin.from_pretrained`] functions.
-
-    For more details, see the original paper: https://arxiv.org/abs/2111.14822
+    This model inherits from [`SchedulerMixin`] and [`ConfigMixin`]. Check the superclass documentation for the generic
+    methods the library implements for all schedulers such as loading and saving.
 
     Args:
         num_vec_classes (`int`):
             The number of classes of the vector embeddings of the latent pixels. Includes the class for the masked
             latent pixel.
-
-        num_train_timesteps (`int`):
-            Number of diffusion steps used to train the model.
-
-        alpha_cum_start (`float`):
+        num_train_timesteps (`int`, defaults to 100):
+            The number of diffusion steps to train the model.
+        alpha_cum_start (`float`, defaults to 0.99999):
             The starting cumulative alpha value.
-
-        alpha_cum_end (`float`):
+        alpha_cum_end (`float`, defaults to 0.00009):
             The ending cumulative alpha value.
-
-        gamma_cum_start (`float`):
+        gamma_cum_start (`float`, defaults to 0.00009):
             The starting cumulative gamma value.
-
-        gamma_cum_end (`float`):
+        gamma_cum_end (`float`, defaults to 0.99999):
             The ending cumulative gamma value.
     """
 
@@ -155,26 +139,21 @@ class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
 
     @register_to_config
     def __init__(
-            self,
-            num_vec_classes: int,
-            num_train_timesteps: int=100,
-            alpha_cum_start: float=0.99999,
-            alpha_cum_end: float=0.000009,
-            gamma_cum_start: float=0.000009,
-            gamma_cum_end: float=0.99999, ):
+        self,
+        num_vec_classes: int,
+        num_train_timesteps: int = 100,
+        alpha_cum_start: float = 0.99999,
+        alpha_cum_end: float = 0.000009,
+        gamma_cum_start: float = 0.000009,
+        gamma_cum_end: float = 0.99999,
+    ):
         self.num_embed = num_vec_classes
 
         # By convention, the index for the mask class is the last class index
         self.mask_class = self.num_embed - 1
 
-        at, att = alpha_schedules(
-            num_train_timesteps,
-            alpha_cum_start=alpha_cum_start,
-            alpha_cum_end=alpha_cum_end)
-        ct, ctt = gamma_schedules(
-            num_train_timesteps,
-            gamma_cum_start=gamma_cum_start,
-            gamma_cum_end=gamma_cum_end)
+        at, att = alpha_schedules(num_train_timesteps, alpha_cum_start=alpha_cum_start, alpha_cum_end=alpha_cum_end)
+        ct, ctt = gamma_schedules(num_train_timesteps, gamma_cum_start=gamma_cum_start, gamma_cum_end=gamma_cum_end)
 
         num_non_mask_classes = self.num_embed - 1
         bt = (1 - at - ct) / num_non_mask_classes
@@ -203,55 +182,50 @@ class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
 
         # setable values
         self.num_inference_steps = None
-        self.timesteps = paddle.to_tensor(
-            np.arange(0, num_train_timesteps)[::-1].copy())
+        self.timesteps = paddle.to_tensor(np.arange(0, num_train_timesteps)[::-1].copy())
 
     def set_timesteps(self, num_inference_steps: int):
         """
-        Sets the discrete timesteps used for the diffusion chain. Supporting function to be run before inference.
+        Sets the discrete timesteps used for the diffusion chain (to be run before inference).
 
         Args:
             num_inference_steps (`int`):
-                the number of diffusion steps used when generating samples with a pre-trained model.
+                The number of diffusion steps used when generating samples with a pre-trained model.
         """
         self.num_inference_steps = num_inference_steps
         timesteps = np.arange(0, self.num_inference_steps)[::-1].copy()
         self.timesteps = paddle.to_tensor(timesteps)
 
     def step(
-            self,
-            model_output: paddle.Tensor,
-            timestep: paddle.Tensor,
-            sample: paddle.Tensor,
-            generator: Optional[Union[paddle.Generator, List[
-                paddle.Generator]]]=None,
-            return_dict: bool=True, ) -> Union[VQDiffusionSchedulerOutput,
-                                               Tuple]:
+        self,
+        model_output: paddle.Tensor,
+        timestep: paddle.int64,
+        sample: paddle.Tensor,
+        generator: Optional[paddle.Generator] = None,
+        return_dict: bool = True,
+    ) -> Union[VQDiffusionSchedulerOutput, Tuple]:
         """
-        Predict the sample at the previous timestep via the reverse transition distribution i.e. Equation (11). See the
-        docstring for `self.q_posterior` for more in depth docs on how Equation (11) is computed.
+        Predict the sample from the previous timestep by the reverse transition distribution. See
+        [`~VQDiffusionScheduler.q_posterior`] for more details about how the distribution is computer.
 
         Args:
             log_p_x_0: (`paddle.Tensor` of shape `(batch size, num classes - 1, num latent pixels)`):
                 The log probabilities for the predicted classes of the initial latent pixels. Does not include a
                 prediction for the masked class as the initial unnoised image cannot be masked.
-
-            t (`paddle.Tensor`):
+            t (`paddle.int64`):
                 The timestep that determines which transition matrices are used.
-
-            x_t: (`paddle.Tensor` of shape `(batch size, num latent pixels)`):
-                The classes of each latent pixel at time `t`
-
-            generator: (`paddle.Generator` or None):
-                RNG for the noise applied to p(x_{t-1} | x_t) before it is sampled from.
-
-            return_dict (`bool`):
-                option for returning tuple rather than VQDiffusionSchedulerOutput class
+            x_t (`paddle.Tensor` of shape `(batch size, num latent pixels)`):
+                The classes of each latent pixel at time `t`.
+            generator (`paddle.Generator`, or `None`):
+                A random number generator for the noise applied to `p(x_{t-1} | x_t)` before it is sampled from.
+            return_dict (`bool`, *optional*, defaults to `True`):
+                Whether or not to return a [`~schedulers.scheduling_vq_diffusion.VQDiffusionSchedulerOutput`] or
+                `tuple`.
 
         Returns:
-            [`~schedulers.scheduling_utils.VQDiffusionSchedulerOutput`] or `tuple`:
-            [`~schedulers.scheduling_utils.VQDiffusionSchedulerOutput`] if `return_dict` is True, otherwise a `tuple`.
-            When returning a tuple, the first element is the sample tensor.
+            [`~schedulers.scheduling_vq_diffusion.VQDiffusionSchedulerOutput`] or `tuple`:
+                If return_dict is `True`, [`~schedulers.scheduling_vq_diffusion.VQDiffusionSchedulerOutput`] is
+                returned, otherwise a tuple is returned where the first element is the sample tensor.
         """
         if timestep == 0:
             log_p_x_t_min_1 = model_output
@@ -263,46 +237,40 @@ class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
         x_t_min_1 = log_p_x_t_min_1.argmax(axis=1)
 
         if not return_dict:
-            return (x_t_min_1, )
+            return (x_t_min_1,)
 
         return VQDiffusionSchedulerOutput(prev_sample=x_t_min_1)
 
     def q_posterior(self, log_p_x_0, x_t, t):
         """
-        Calculates the log probabilities for the predicted classes of the image at timestep `t-1`. I.e. Equation (11).
-
-        Instead of directly computing equation (11), we use Equation (5) to restate Equation (11) in terms of only
-        forward probabilities.
-
-        Equation (11) stated in terms of forward probabilities via Equation (5):
-
-        Where:
-        - the sum is over x_0 = {C_0 ... C_{k-1}} (classes for x_0)
+        Calculates the log probabilities for the predicted classes of the image at timestep `t-1`:
 
+        ```
         p(x_{t-1} | x_t) = sum( q(x_t | x_{t-1}) * q(x_{t-1} | x_0) * p(x_0) / q(x_t | x_0) )
+        ```
 
         Args:
-            log_p_x_0: (`paddle.Tensor` of shape `(batch size, num classes - 1, num latent pixels)`):
+            log_p_x_0 (`paddle.Tensor` of shape `(batch size, num classes - 1, num latent pixels)`):
                 The log probabilities for the predicted classes of the initial latent pixels. Does not include a
                 prediction for the masked class as the initial unnoised image cannot be masked.
-
-            x_t: (`paddle.Tensor` of shape `(batch size, num latent pixels)`):
-                The classes of each latent pixel at time `t`
-
-            t (paddle.Tensor):
+            x_t (`paddle.Tensor` of shape `(batch size, num latent pixels)`):
+                The classes of each latent pixel at time `t`.
+            t (`torch.Long`):
                 The timestep that determines which transition matrix is used.
 
         Returns:
             `paddle.Tensor` of shape `(batch size, num classes, num latent pixels)`:
-                The log probabilities for the predicted classes of the image at timestep `t-1`. I.e. Equation (11).
+                The log probabilities for the predicted classes of the image at timestep `t-1`.
         """
         log_onehot_x_t = index_to_log_onehot(x_t, self.num_embed)
 
         log_q_x_t_given_x_0 = self.log_Q_t_transitioning_to_known_class(
-            t=t, x_t=x_t, log_onehot_x_t=log_onehot_x_t, cumulative=True)
+            t=t, x_t=x_t, log_onehot_x_t=log_onehot_x_t, cumulative=True
+        )
 
         log_q_t_given_x_t_min_1 = self.log_Q_t_transitioning_to_known_class(
-            t=t, x_t=x_t, log_onehot_x_t=log_onehot_x_t, cumulative=False)
+            t=t, x_t=x_t, log_onehot_x_t=log_onehot_x_t, cumulative=False
+        )
 
         # p_0(x_0=C_0 | x_t) / q(x_t | x_0=C_0)          ...      p_n(x_0=C_0 | x_t) / q(x_t | x_0=C_0)
         #               .                    .                                   .
@@ -384,32 +352,23 @@ class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
         # The last row is trivially verified. The other rows can be verified by directly expanding equation (11) stated in terms of forward probabilities.
         return log_p_x_t_min_1
 
-    def log_Q_t_transitioning_to_known_class(self,
-                                             *,
-                                             t: paddle.Tensor,
-                                             x_t: paddle.Tensor,
-                                             log_onehot_x_t: paddle.Tensor,
-                                             cumulative: bool):
+    def log_Q_t_transitioning_to_known_class(
+        self, *, t: paddle.Tensor, x_t: paddle.Tensor, log_onehot_x_t: paddle.Tensor, cumulative: bool
+    ):
         """
-        Returns the log probabilities of the rows from the (cumulative or non-cumulative) transition matrix for each
+        Calculates the log probabilities of the rows from the (cumulative or non-cumulative) transition matrix for each
         latent pixel in `x_t`.
 
-        See equation (7) for the complete non-cumulative transition matrix. The complete cumulative transition matrix
-        is the same structure except the parameters (alpha, beta, gamma) are the cumulative analogs.
-
         Args:
             t (paddle.Tensor):
                 The timestep that determines which transition matrix is used.
-
             x_t (`paddle.Tensor` of shape `(batch size, num latent pixels)`):
                 The classes of each latent pixel at time `t`.
-
             log_onehot_x_t (`paddle.Tensor` of shape `(batch size, num classes, num latent pixels)`):
-                The log one-hot vectors of `x_t`
-
+                The log one-hot vectors of `x_t`.
             cumulative (`bool`):
-                If cumulative is `False`, we use the single step transition matrix `t-1`->`t`. If cumulative is `True`,
-                we use the cumulative transition matrix `0`->`t`.
+                If cumulative is `False`, the single step transition matrix `t-1`->`t` is used. If cumulative is
+                `True`, the cumulative transition matrix `0`->`t` is used.
 
         Returns:
             `paddle.Tensor` of shape `(batch size, num classes - 1, num latent pixels)`:
@@ -462,9 +421,7 @@ class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
             #
             # `P(x_t=mask|x_{t-1=mask}) = 1` and 1 will be the value of the last row of the onehot vector
             # if x_t is masked
-            log_onehot_x_t_transitioning_from_masked = log_onehot_x_t[:,
-                                                                      -1, :].unsqueeze(
-                                                                          1)
+            log_onehot_x_t_transitioning_from_masked = log_onehot_x_t[:, -1, :].unsqueeze(1)
 
         # `index_to_log_onehot` will add onehot vectors for masked pixels,
         # so the default one hot matrix has one too many rows. See the doc string
@@ -486,14 +443,12 @@ class VQDiffusionScheduler(SchedulerMixin, ConfigMixin):
 
         # The whole column of each masked pixel is `c`
         mask_class_mask = x_t == self.mask_class
-        mask_class_mask = mask_class_mask.unsqueeze(1).expand(
-            [-1, self.num_embed - 1, -1])
+        mask_class_mask = mask_class_mask.unsqueeze(1).expand([-1, self.num_embed - 1, -1])
         # log_Q_t[mask_class_mask] = c
         log_Q_t = paddle.where(mask_class_mask, c, log_Q_t)
 
         if not cumulative:
-            log_Q_t = paddle.concat(
-                (log_Q_t, log_onehot_x_t_transitioning_from_masked), axis=1)
+            log_Q_t = paddle.concat((log_Q_t, log_onehot_x_t_transitioning_from_masked), axis=1)
 
         return log_Q_t
 
