@@ -15,7 +15,6 @@
 
 import contextlib
 import copy
-import os
 import random
 from typing import Any, Dict, Optional, Union
 
@@ -24,28 +23,12 @@ import paddle
 import paddle.distributed
 import paddle.nn as nn
 
+from .models import UNet2DConditionModel
 from .utils import deprecate, get_logger
 
 logger = get_logger(__name__)
 
 
-def enable_full_determinism(seed: int):
-    """
-    Helper function for reproducible behavior during distributed training.
-    """
-    # set seed first
-    set_seed(seed)
-
-    #  Enable Paddle deterministic mode. This potentially requires either the environment
-    #  variable 'CUDA_LAUNCH_BLOCKING' or 'CUBLAS_WORKSPACE_CONFIG' to be set,
-    # depending on the CUDA version, so we set them both here
-    os.environ["CUDA_LAUNCH_BLOCKING"] = "1"
-    os.environ["CUBLAS_WORKSPACE_CONFIG"] = ":16:8"
-    os.environ["FLAGS_cudnn_deterministic"] = "True"
-    os.environ["FLAGS_benchmark"] = "True"
-    os.environ["FLAGS_conv_workspace_size_limit"] = "4096"
-
-
 def set_seed(seed: int):
     """
     Args:
@@ -59,24 +42,70 @@ def set_seed(seed: int):
     # ^^ safe to call this function even if cuda is not available
 
 
-    # Adapted from torch-ema https://github.com/fadel/pytorch_ema/blob/master/torch_ema/ema.py#L14
+def compute_snr(noise_scheduler, timesteps):
+    """
+    Computes SNR as per
+    https://github.com/TiankaiHang/Min-SNR-Diffusion-Training/blob/521b624bd70c67cee4bdf49225915f5945a872e3/guided_diffusion/gaussian_diffusion.py#L847-L849
+    """
+    alphas_cumprod = noise_scheduler.alphas_cumprod
+    sqrt_alphas_cumprod = alphas_cumprod**0.5
+    sqrt_one_minus_alphas_cumprod = (1.0 - alphas_cumprod) ** 0.5
+
+    # Expand the tensors.
+    # Adapted from https://github.com/TiankaiHang/Min-SNR-Diffusion-Training/blob/521b624bd70c67cee4bdf49225915f5945a872e3/guided_diffusion/gaussian_diffusion.py#L1026
+    sqrt_alphas_cumprod = sqrt_alphas_cumprod[timesteps].cast("float32")
+    while len(sqrt_alphas_cumprod.shape) < len(timesteps.shape):
+        sqrt_alphas_cumprod = sqrt_alphas_cumprod[..., None]
+    alpha = sqrt_alphas_cumprod.expand(timesteps.shape)
+
+    sqrt_one_minus_alphas_cumprod = sqrt_one_minus_alphas_cumprod[timesteps].cast("float32")
+    while len(sqrt_one_minus_alphas_cumprod.shape) < len(timesteps.shape):
+        sqrt_one_minus_alphas_cumprod = sqrt_one_minus_alphas_cumprod[..., None]
+    sigma = sqrt_one_minus_alphas_cumprod.expand(timesteps.shape)
+
+    # Compute SNR.
+    snr = (alpha / sigma) ** 2
+    return snr
+
+
+def unet_lora_state_dict(unet: UNet2DConditionModel) -> Dict[str, paddle.Tensor]:
+    r"""
+    Returns:
+        A state dict containing just the LoRA parameters.
+    """
+    lora_state_dict = {}
+
+    for name, module in unet.named_sublayers(include_self=True):
+        if hasattr(module, "set_lora_layer"):
+            lora_layer = getattr(module, "lora_layer")
+            if lora_layer is not None:
+                current_lora_layer_sd = lora_layer.state_dict()
+                for lora_layer_matrix_name, lora_param in current_lora_layer_sd.items():
+                    # The matrix name can either be "down" or "up".
+                    lora_state_dict[f"unet.{name}.lora.{lora_layer_matrix_name}"] = lora_param
+
+    return lora_state_dict
+
+
+# Adapted from torch-ema https://github.com/fadel/pytorch_ema/blob/master/torch_ema/ema.py#L14
 class EMAModel:
     """
     Exponential Moving Average of models weights
     """
 
     def __init__(
-            self,
-            parameters,
-            decay: float=0.9999,
-            min_decay: float=0.0,
-            update_after_step: int=0,
-            use_ema_warmup: bool=False,
-            inv_gamma: Union[float, int]=1.0,
-            power: Union[float, int]=2 / 3,
-            model_cls: Optional[Any]=None,
-            model_config: Dict[str, Any]=None,
-            **kwargs, ):
+        self,
+        parameters,
+        decay: float = 0.9999,
+        min_decay: float = 0.0,
+        update_after_step: int = 0,
+        use_ema_warmup: bool = False,
+        inv_gamma: Union[float, int] = 1.0,
+        power: Union[float, int] = 2 / 3,
+        model_cls: Optional[Any] = None,
+        model_config: Dict[str, Any] = None,
+        **kwargs,
+    ):
         """
         Args:
             parameters (Iterable[nn.Parameter]): The parameters to track.
@@ -98,12 +127,14 @@ class EMAModel:
         if isinstance(parameters, nn.Layer):
             deprecation_message = (
                 "Passing a `nn.Layer` to `ExponentialMovingAverage` is deprecated. "
-                "Please pass the parameters of the module instead.")
+                "Please pass the parameters of the module instead."
+            )
             deprecate(
                 "passing a `nn.Layer` to `ExponentialMovingAverage`",
                 "1.0.0",
                 deprecation_message,
-                standard_warn=False, )
+                standard_warn=False,
+            )
             parameters = parameters.parameters()
 
             # set use_ema_warmup to True if a nn.Layer is passed for backwards compatibility
@@ -111,14 +142,12 @@ class EMAModel:
 
         if kwargs.get("max_value", None) is not None:
             deprecation_message = "The `max_value` argument is deprecated. Please use `decay` instead."
-            deprecate(
-                "max_value", "1.0.0", deprecation_message, standard_warn=False)
+            deprecate("max_value", "1.0.0", deprecation_message, standard_warn=False)
             decay = kwargs["max_value"]
 
         if kwargs.get("min_value", None) is not None:
             deprecation_message = "The `min_value` argument is deprecated. Please use `min_decay` instead."
-            deprecate(
-                "min_value", "1.0.0", deprecation_message, standard_warn=False)
+            deprecate("min_value", "1.0.0", deprecation_message, standard_warn=False)
             min_decay = kwargs["min_value"]
 
         parameters = list(parameters)
@@ -126,8 +155,7 @@ class EMAModel:
 
         if kwargs.get("device", None) is not None:
             deprecation_message = "The `device` argument is deprecated. Please use `to` instead."
-            deprecate(
-                "device", "1.0.0", deprecation_message, standard_warn=False)
+            deprecate("device", "1.0.0", deprecation_message, standard_warn=False)
             self.to(device=kwargs["device"])
 
         self.temp_stored_params = None
@@ -149,23 +177,17 @@ class EMAModel:
         _, ema_kwargs = model_cls.load_config(path, return_unused_kwargs=True)
         model = model_cls.from_pretrained(path)
 
-        ema_model = cls(model.parameters(),
-                        model_cls=model_cls,
-                        model_config=model.config)
+        ema_model = cls(model.parameters(), model_cls=model_cls, model_config=model.config)
 
         ema_model.load_state_dict(ema_kwargs)
         return ema_model
 
     def save_pretrained(self, path):
         if self.model_cls is None:
-            raise ValueError(
-                "`save_pretrained` can only be used if `model_cls` was defined at __init__."
-            )
+            raise ValueError("`save_pretrained` can only be used if `model_cls` was defined at __init__.")
 
         if self.model_config is None:
-            raise ValueError(
-                "`save_pretrained` can only be used if `model_config` was defined at __init__."
-            )
+            raise ValueError("`save_pretrained` can only be used if `model_config` was defined at __init__.")
 
         model = self.model_cls.from_config(self.model_config)
         state_dict = self.state_dict()
@@ -179,14 +201,14 @@ class EMAModel:
         """
         Compute the decay factor for the exponential moving average.
         """
-        # we donot -1!
+        # NOTE we donot -1! (yujun06) diffirent from diffusers's implementation
         step = max(0, optimization_step - self.update_after_step)
 
         if step <= 0:
             return 0.0
 
         if self.use_ema_warmup:
-            cur_decay_value = 1 - (1 + step / self.inv_gamma)**-self.power
+            cur_decay_value = 1 - (1 + step / self.inv_gamma) ** -self.power
         else:
             cur_decay_value = (1 + step) / (10 + step)
 
@@ -200,12 +222,14 @@ class EMAModel:
         if isinstance(parameters, nn.Layer):
             deprecation_message = (
                 "Passing a `nn.Layer` to `ExponentialMovingAverage.step` is deprecated. "
-                "Please pass the parameters of the module instead.")
+                "Please pass the parameters of the module instead."
+            )
             deprecate(
                 "passing a `nn.Layer` to `ExponentialMovingAverage.step`",
                 "1.0.0",
                 deprecation_message,
-                standard_warn=False, )
+                standard_warn=False,
+            )
             parameters = parameters.parameters()
 
         parameters = list(parameters)
@@ -217,12 +241,14 @@ class EMAModel:
         self.cur_decay_value = decay
         one_minus_decay = 1 - decay
 
+        context_manager = contextlib.nullcontext
+
         for s_param, param in zip(self.shadow_params, parameters):
-            if not param.stop_gradient:
-                s_param.copy_(s_param - one_minus_decay * (s_param - param),
-                              True)
-            else:
-                s_param.copy_(param, True)
+            with context_manager():
+                if not param.stop_gradient:
+                    s_param.copy_(s_param - one_minus_decay * (s_param - param.cast(s_param.dtype)), False)
+                else:
+                    s_param.copy_(param.cast(s_param.dtype), False)
 
     def copy_to(self, parameters) -> None:
         """
@@ -235,7 +261,19 @@ class EMAModel:
         """
         parameters = list(parameters)
         for s_param, param in zip(self.shadow_params, parameters):
-            param.copy_(s_param, True)
+            param.copy_(s_param.cast(param.dtype), False)
+
+    def to(self, device=None, dtype=None) -> None:
+        r"""Move internal buffers of the ExponentialMovingAverage to `device`.
+
+        Args:
+            device: like `device` argument to `paddle.Tensor.to`
+        """
+        # .to() on the tensors handles None correctly
+        self.shadow_params = [
+            p._to(device=device, dtype=dtype) if p.is_floating_point() else p._to(device=device)
+            for p in self.shadow_params
+        ]
 
     def state_dict(self) -> dict:
         r"""
@@ -263,9 +301,7 @@ class EMAModel:
             parameters: Iterable of `nn.Parameter`; the parameters to be
                 temporarily stored.
         """
-        self.temp_stored_params = [
-            param.detach().cpu().clone() for param in parameters
-        ]
+        self.temp_stored_params = [param.detach().cpu().clone() for param in parameters]
 
     def restore(self, parameters) -> None:
         r"""
@@ -278,11 +314,9 @@ class EMAModel:
                 `ExponentialMovingAverage` was initialized will be used.
         """
         if self.temp_stored_params is None:
-            raise RuntimeError(
-                "This ExponentialMovingAverage has no `store()`ed weights "
-                "to `restore()`")
+            raise RuntimeError("This ExponentialMovingAverage has no `store()`ed weights " "to `restore()`")
         for c_param, param in zip(self.temp_stored_params, parameters):
-            param.copy_(c_param, True)
+            param.copy_(c_param.cast(param.dtype), False)
 
         # Better memory-wise.
         self.temp_stored_params = None
@@ -306,18 +340,15 @@ class EMAModel:
         if not isinstance(self.min_decay, float):
             raise ValueError("Invalid min_decay")
 
-        self.optimization_step = state_dict.get("optimization_step",
-                                                self.optimization_step)
+        self.optimization_step = state_dict.get("optimization_step", self.optimization_step)
         if not isinstance(self.optimization_step, int):
             raise ValueError("Invalid optimization_step")
 
-        self.update_after_step = state_dict.get("update_after_step",
-                                                self.update_after_step)
+        self.update_after_step = state_dict.get("update_after_step", self.update_after_step)
         if not isinstance(self.update_after_step, int):
             raise ValueError("Invalid update_after_step")
 
-        self.use_ema_warmup = state_dict.get("use_ema_warmup",
-                                             self.use_ema_warmup)
+        self.use_ema_warmup = state_dict.get("use_ema_warmup", self.use_ema_warmup)
         if not isinstance(self.use_ema_warmup, bool):
             raise ValueError("Invalid use_ema_warmup")
 
@@ -334,8 +365,7 @@ class EMAModel:
             self.shadow_params = shadow_params
             if not isinstance(self.shadow_params, list):
                 raise ValueError("shadow_params must be a list")
-            if not all(
-                    isinstance(p, paddle.Tensor) for p in self.shadow_params):
+            if not all(paddle.is_tensor(p) for p in self.shadow_params):
                 raise ValueError("shadow_params must all be Tensors")
 
 
@@ -349,17 +379,13 @@ def main_process_first(desc="work"):
         try:
             if not is_main_process:
                 # tell all replicas to wait
-                logger.debug(
-                    f"{rank}: waiting for the {main_process_desc} to perform {desc}"
-                )
+                logger.debug(f"{rank}: waiting for the {main_process_desc} to perform {desc}")
                 paddle.distributed.barrier()
             yield
         finally:
             if is_main_process:
                 # the wait is over
-                logger.debug(
-                    f"{rank}: {main_process_desc} completed {desc}, releasing all replicas"
-                )
+                logger.debug(f"{rank}: {main_process_desc} completed {desc}, releasing all replicas")
                 paddle.distributed.barrier()
     else:
         yield
