@@ -37,24 +37,44 @@ __all__ = [
     "reset_initialized_parameter",
 ]
 
+try:
+    from paddle.nn.initializer.lazy_init import lazy_init_helper
+except ImportError:
+
+    class lazy_init_helper:
+        _state = False
+
 
 def _no_grad_uniform_(tensor, a, b):
-    with paddle.no_grad():
-        tensor.uniform_(min=a, max=b)
+    if lazy_init_helper()._state:
+        return tensor
+    try:
+        with paddle.no_grad():
+            tensor.uniform_(min=a, max=b)
+    except:
+        pass
     return tensor
 
 
 def _no_grad_normal_(tensor, mean=0.0, std=1.0):
-    with paddle.no_grad():
-        tensor.copy_(
-            paddle.normal(
-                mean=mean, std=std, shape=tensor.shape), True)
+    if lazy_init_helper()._state:
+        return tensor
+    try:
+        with paddle.no_grad():
+            tensor.copy_(paddle.normal(mean=mean, std=std, shape=tensor.shape), False)
+    except:
+        pass
     return tensor
 
 
 def _no_grad_fill_(tensor, value=0.0):
-    with paddle.no_grad():
-        tensor.fill_(value)
+    if lazy_init_helper()._state:
+        return tensor
+    try:
+        with paddle.no_grad():
+            tensor.fill_(value)
+    except:
+        pass
     return tensor
 
 
@@ -119,8 +139,13 @@ def zeros_(tensor):
 
 
 def vector_(tensor, vector):
-    with paddle.no_grad():
-        tensor.set_value(paddle.to_tensor(vector, dtype=tensor.dtype))
+    if lazy_init_helper()._state:
+        return tensor
+    try:
+        with paddle.no_grad():
+            tensor.set_value(paddle.to_tensor(vector, dtype=tensor.dtype))
+    except:
+        pass
     return tensor
 
 
@@ -134,9 +159,7 @@ def _calculate_fan_in_and_fan_out(tensor, reverse=False):
         Tuple[fan_in, fan_out]
     """
     if tensor.ndim < 2:
-        raise ValueError(
-            "Fan in and fan out can not be computed for tensor with fewer than 2 dimensions"
-        )
+        raise ValueError("Fan in and fan out can not be computed for tensor with fewer than 2 dimensions")
 
     if reverse:
         num_input_fmaps, num_output_fmaps = tensor.shape[0], tensor.shape[1]
@@ -163,6 +186,8 @@ def xavier_uniform_(tensor, gain=1.0, reverse=False):
     Return:
         tensor
     """
+    if lazy_init_helper()._state:
+        return tensor
     fan_in, fan_out = _calculate_fan_in_and_fan_out(tensor, reverse=reverse)
     std = gain * math.sqrt(2.0 / float(fan_in + fan_out))
     k = math.sqrt(3.0) * std
@@ -179,6 +204,8 @@ def xavier_normal_(tensor, gain=1.0, reverse=False):
     Return:
         tensor
     """
+    if lazy_init_helper()._state:
+        return tensor
     fan_in, fan_out = _calculate_fan_in_and_fan_out(tensor, reverse=reverse)
     std = gain * math.sqrt(2.0 / float(fan_in + fan_out))
     return _no_grad_normal_(tensor, 0, std)
@@ -189,8 +216,7 @@ def _calculate_correct_fan(tensor, mode, reverse=False):
     mode = mode.lower()
     valid_modes = ["fan_in", "fan_out"]
     if mode not in valid_modes:
-        raise ValueError("Mode {} not supported, please use one of {}".format(
-            mode, valid_modes))
+        raise ValueError("Mode {} not supported, please use one of {}".format(mode, valid_modes))
 
     fan_in, fan_out = _calculate_fan_in_and_fan_out(tensor, reverse)
 
@@ -198,10 +224,7 @@ def _calculate_correct_fan(tensor, mode, reverse=False):
 
 
 def _calculate_gain(nonlinearity, param=None):
-    linear_fns = [
-        "linear", "conv1d", "conv2d", "conv3d", "conv_transpose1d",
-        "conv_transpose2d", "conv_transpose3d"
-    ]
+    linear_fns = ["linear", "conv1d", "conv2d", "conv3d", "conv_transpose1d", "conv_transpose2d", "conv_transpose3d"]
     if nonlinearity in linear_fns or nonlinearity == "sigmoid":
         return 1
     elif nonlinearity == "tanh":
@@ -211,13 +234,11 @@ def _calculate_gain(nonlinearity, param=None):
     elif nonlinearity == "leaky_relu":
         if param is None:
             negative_slope = 0.01
-        elif not isinstance(param, bool) and isinstance(
-                param, int) or isinstance(param, float):
+        elif not isinstance(param, bool) and isinstance(param, int) or isinstance(param, float):
             # True/False are instances of int, hence check above
             negative_slope = param
         else:
-            raise ValueError("negative_slope {} not a valid number".format(
-                param))
+            raise ValueError("negative_slope {} not a valid number".format(param))
         return math.sqrt(2.0 / (1 + negative_slope**2))
     elif nonlinearity == "selu":
         return 3.0 / 4
@@ -225,11 +246,7 @@ def _calculate_gain(nonlinearity, param=None):
         raise ValueError("Unsupported nonlinearity {}".format(nonlinearity))
 
 
-def kaiming_uniform_(tensor,
-                     a=0,
-                     mode="fan_in",
-                     nonlinearity="leaky_relu",
-                     reverse=False):
+def kaiming_uniform_(tensor, a=0, mode="fan_in", nonlinearity="leaky_relu", reverse=False):
     """
     Modified tensor inspace using kaiming_uniform method
     Args:
@@ -240,6 +257,8 @@ def kaiming_uniform_(tensor,
     Return:
         tensor
     """
+    if lazy_init_helper()._state:
+        return tensor
     fan = _calculate_correct_fan(tensor, mode, reverse)
     gain = _calculate_gain(nonlinearity, a)
     std = gain / math.sqrt(fan)
@@ -247,11 +266,7 @@ def kaiming_uniform_(tensor,
     return _no_grad_uniform_(tensor, -k, k)
 
 
-def kaiming_normal_(tensor,
-                    a=0,
-                    mode="fan_in",
-                    nonlinearity="leaky_relu",
-                    reverse=False):
+def kaiming_normal_(tensor, a=0, mode="fan_in", nonlinearity="leaky_relu", reverse=False):
     """
     Modified tensor inspace using kaiming_normal_
     Args:
@@ -262,6 +277,8 @@ def kaiming_normal_(tensor,
     Return:
         tensor
     """
+    if lazy_init_helper()._state:
+        return tensor
     fan = _calculate_correct_fan(tensor, mode, reverse)
     gain = _calculate_gain(nonlinearity, a)
     std = gain / math.sqrt(fan)
@@ -269,12 +286,16 @@ def kaiming_normal_(tensor,
 
 
 def linear_init_(module):
+    if lazy_init_helper()._state:
+        return
     bound = 1 / math.sqrt(module.weight.shape[0])
     uniform_(module.weight, -bound, bound)
     uniform_(module.bias, -bound, bound)
 
 
 def conv_init_(module):
+    if lazy_init_helper()._state:
+        return
     bound = 1 / np.sqrt(np.prod(module.weight.shape[1:]))
     uniform_(module.weight, -bound, bound)
     if module.bias is not None:
@@ -297,10 +318,11 @@ def reset_initialized_parameter(model, include_self=True):
     Return:
         None
     """
+    if lazy_init_helper()._state:
+        return
     for _, m in model.named_sublayers(include_self=include_self):
         if isinstance(m, nn.Conv2D):
-            k = float(m._groups) / (m._in_channels * m._kernel_size[0] *
-                                    m._kernel_size[1])
+            k = float(m._groups) / (m._in_channels * m._kernel_size[0] * m._kernel_size[1])
             k = math.sqrt(k)
             _no_grad_uniform_(m.weight, -k, k)
             if hasattr(m, "bias") and getattr(m, "bias") is not None:
@@ -325,17 +347,17 @@ def reset_initialized_parameter(model, include_self=True):
 class Init:
     def __init__(self):
         for init_func in [
-                uniform_,
-                normal_,
-                constant_,
-                ones_,
-                zeros_,
-                xavier_uniform_,
-                xavier_normal_,
-                kaiming_uniform_,
-                kaiming_normal_,
-                linear_init_,
-                conv_init_,
+            uniform_,
+            normal_,
+            constant_,
+            ones_,
+            zeros_,
+            xavier_uniform_,
+            xavier_normal_,
+            kaiming_uniform_,
+            kaiming_normal_,
+            linear_init_,
+            conv_init_,
         ]:
             setattr(self, init_func.__name__, init_func)
 
