@@ -1,5 +1,6 @@
+# coding=utf-8
 # Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
-# Copyright 2023 The HuggingFace Team. All rights reserved.
+# Copyright 2023 HuggingFace Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +19,7 @@ import unittest
 import paddle
 
 from ppdiffusers import UNet1DModel
-from ppdiffusers.utils import floats_tensor, slow
+from ppdiffusers.utils.testing_utils import floats_tensor, slow
 
 from .test_modeling_common import ModelTesterMixin, UNetTesterMixin
 
@@ -32,17 +33,19 @@ class UNet1DModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
         batch_size = 4
         num_features = 14
         seq_len = 16
+
         noise = floats_tensor((batch_size, num_features, seq_len))
         time_step = paddle.to_tensor([10] * batch_size)
+
         return {"sample": noise, "timestep": time_step}
 
     @property
     def input_shape(self):
-        return 4, 14, 16
+        return (4, 14, 16)
 
     @property
     def output_shape(self):
-        return 4, 14, 16
+        return (4, 14, 16)
 
     def test_ema_training(self):
         pass
@@ -59,6 +62,9 @@ class UNet1DModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
     def test_from_save_pretrained(self):
         super().test_from_save_pretrained()
 
+    def test_from_save_pretrained_variant(self):
+        super().test_from_save_pretrained_variant()
+
     def test_model_from_pretrained(self):
         super().test_model_from_pretrained()
 
@@ -78,7 +84,7 @@ class UNet1DModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
             "mid_block_type": "MidResTemporalBlock1D",
             "down_block_types": ("DownResnetBlock1D", "DownResnetBlock1D", "DownResnetBlock1D", "DownResnetBlock1D"),
             "up_block_types": ("UpResnetBlock1D", "UpResnetBlock1D", "UpResnetBlock1D"),
-            "act_fn": "mish",
+            "act_fn": "swish",  # mish
         }
         inputs_dict = self.dummy_input
         return init_dict, inputs_dict
@@ -89,19 +95,27 @@ class UNet1DModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
         )
         self.assertIsNotNone(model)
         self.assertEqual(len(loading_info["missing_keys"]), 0)
+
         image = model(**self.dummy_input)
+
         assert image is not None, "Make sure output is not None"
 
     def test_output_pretrained(self):
         model = UNet1DModel.from_pretrained("bglick13/hopper-medium-v2-value-function-hor32", subfolder="unet")
         paddle.seed(0)
+
         num_features = model.config.in_channels
         seq_len = 16
-        noise = paddle.randn(shape=(1, seq_len, num_features)).transpose(perm=[0, 2, 1])
-        time_step = paddle.full(shape=(num_features,), fill_value=0)
+        noise = paddle.randn((1, seq_len, num_features)).transpose(
+            (0, 2, 1)
+        )  # match original, we can update values and remove
+        time_step = paddle.full((num_features,), 0)
+
         with paddle.no_grad():
-            output = model(noise, time_step).sample.permute(0, 2, 1)
+            output = model(noise, time_step).sample.transpose((0, 2, 1))
+
         output_slice = output[0, -3:, -3:].flatten()
+        # fmt: off
         expected_output_slice = paddle.to_tensor(
             [
                 -0.2857576608657837,
@@ -115,27 +129,32 @@ class UNet1DModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
                 -0.17421625554561615,
             ]
         )
-        self.assertTrue(paddle.allclose(output_slice, expected_output_slice, rtol=0.001))
+        # fmt: on
+        self.assertTrue(paddle.allclose(output_slice, expected_output_slice, rtol=1e-3))
 
     def test_forward_with_norm_groups(self):
+        # Not implemented yet for this UNet
         pass
 
-    # TODO, check this why not pass
     @slow
     def test_unet_1d_maestro(self):
         model_id = "harmonai/maestro-150k"
         model = UNet1DModel.from_pretrained(model_id, subfolder="unet")
+
         sample_size = 65536
         noise = paddle.sin(
             x=paddle.arange(start=sample_size, dtype=paddle.float32)[None, None, :].tile(repeat_times=[1, 2, 1])
         )
-        timestep = paddle.to_tensor([1.0])  # must cast float32
+        timestep = paddle.to_tensor([1.0])
+
         with paddle.no_grad():
             output = model(noise, timestep).sample
+
         output_sum = output.abs().sum()
         output_max = output.abs().max()
+
         assert (output_sum - 224.0896).abs() < 0.5
-        assert (output_max - 0.0607).abs() < 0.0004
+        assert (output_max - 0.0607).abs() < 4e-4
 
 
 class UNetRLModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
@@ -147,17 +166,19 @@ class UNetRLModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
         batch_size = 4
         num_features = 14
         seq_len = 16
+
         noise = floats_tensor((batch_size, num_features, seq_len))
         time_step = paddle.to_tensor([10] * batch_size)
+
         return {"sample": noise, "timestep": time_step}
 
     @property
     def input_shape(self):
-        return 4, 14, 16
+        return (4, 14, 16)
 
     @property
     def output_shape(self):
-        return 4, 14, 1
+        return (4, 14, 1)
 
     def test_determinism(self):
         super().test_determinism()
@@ -168,17 +189,24 @@ class UNetRLModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
     def test_from_save_pretrained(self):
         super().test_from_save_pretrained()
 
+    def test_from_save_pretrained_variant(self):
+        super().test_from_save_pretrained_variant()
+
     def test_model_from_pretrained(self):
         super().test_model_from_pretrained()
 
     def test_output(self):
+        # UNetRL is a value-function is different output shape
         init_dict, inputs_dict = self.prepare_init_args_and_inputs_for_common()
         model = self.model_class(**init_dict)
         model.eval()
+
         with paddle.no_grad():
             output = model(**inputs_dict)
+
             if isinstance(output, dict):
                 output = output.sample
+
         self.assertIsNotNone(output)
         expected_shape = [inputs_dict["sample"].shape[0], 1]
         self.assertEqual(output.shape, expected_shape, "Input and output shapes do not match")
@@ -204,7 +232,7 @@ class UNetRLModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
             "freq_shift": 1.0,
             "flip_sin_to_cos": False,
             "time_embedding_type": "positional",
-            "act_fn": "swish",
+            "act_fn": "mish",  # swish
         }
         inputs_dict = self.dummy_input
         return init_dict, inputs_dict
@@ -215,7 +243,9 @@ class UNetRLModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
         )
         self.assertIsNotNone(value_function)
         self.assertEqual(len(vf_loading_info["missing_keys"]), 0)
+
         image = value_function(**self.dummy_input)
+
         assert image is not None, "Make sure output is not None"
 
     def test_output_pretrained(self):
@@ -223,14 +253,22 @@ class UNetRLModelTests(ModelTesterMixin, UNetTesterMixin, unittest.TestCase):
             "bglick13/hopper-medium-v2-value-function-hor32", output_loading_info=True, subfolder="value_function"
         )
         paddle.seed(0)
+
         num_features = value_function.config.in_channels
         seq_len = 14
-        noise = paddle.randn(shape=(1, seq_len, num_features)).transpose(perm=[0, 2, 1])
-        time_step = paddle.full(shape=(num_features,), fill_value=0)
+        noise = paddle.randn((1, seq_len, num_features)).transpose(
+            (0, 2, 1)
+        )  # match original, we can update values and remove
+        time_step = paddle.full((num_features,), 0)
+
         with paddle.no_grad():
             output = value_function(noise, time_step).sample
+
+        # fmt: off
         expected_output_slice = paddle.to_tensor([291.51135254] * seq_len)
-        self.assertTrue(paddle.allclose(output.squeeze(-1), expected_output_slice, rtol=0.001))
+        # fmt: on
+        self.assertTrue(paddle.allclose(output.squeeze(-1), expected_output_slice, rtol=1e-3))
 
     def test_forward_with_norm_groups(self):
+        # Not implemented yet for this UNet
         pass
