@@ -77,40 +77,42 @@ class UNet2DConditionModelTrainingTests(unittest.TestCase):
 
     @slow
     def test_training_step_equality(self):
-        ddpm_scheduler = DDPMScheduler(
-            num_train_timesteps=1000, beta_start=0.00085, beta_end=0.012, beta_schedule="scaled_linear"
-        )
-        ddim_scheduler = DDIMScheduler(
-            num_train_timesteps=1000, beta_start=0.00085, beta_end=0.012, beta_schedule="scaled_linear"
-        )
-        assert ddpm_scheduler.config.num_train_timesteps == ddim_scheduler.config.num_train_timesteps
-        set_seed(0)
-        clean_images = [paddle.randn(shape=(4, 3, 32, 32)).clip(min=-1, max=1) for _ in range(4)]
-        noise = [paddle.randn(shape=(4, 3, 32, 32)) for _ in range(4)]
-        text_embeddings = [paddle.randn(shape=(4, 77, 1280)) for _ in range(4)]
-        timesteps = [paddle.randint(0, 1000, (4,)).astype(dtype="int64") for _ in range(4)]
-        model, optimizer = self.get_model_optimizer(resolution=32)
-        model.train()
-        for i in range(4):
-            optimizer.clear_grad()
-            ddpm_noisy_images = ddpm_scheduler.add_noise(clean_images[i], noise[i], timesteps[i])
-            ddpm_noise_pred = model(ddpm_noisy_images, timesteps[i], encoder_hidden_states=text_embeddings[i]).sample
-            loss = paddle.nn.functional.mse_loss(input=ddpm_noise_pred, label=noise[i])
-            loss.backward()
-            optimizer.step()
-        del model, optimizer
-        model, optimizer = self.get_model_optimizer(resolution=32)
-        model.train()
-        for i in range(4):
-            optimizer.clear_grad()
-            ddim_noisy_images = ddim_scheduler.add_noise(clean_images[i], noise[i], timesteps[i])
-            ddim_noise_pred = model(ddim_noisy_images, timesteps[i], encoder_hidden_states=text_embeddings[i]).sample
-            loss = paddle.nn.functional.mse_loss(input=ddim_noise_pred, label=noise[i])
-            loss.backward()
-            optimizer.step()
-        del model, optimizer
-        self.assertTrue(paddle.allclose(ddpm_noisy_images, ddim_noisy_images, atol=1e-05))
-        self.assertTrue(paddle.allclose(ddpm_noise_pred, ddim_noise_pred, atol=1e-04))
+        pass
+        # this will OOM on V100 32G GPU
+        # ddpm_scheduler = DDPMScheduler(
+        #     num_train_timesteps=1000, beta_start=0.00085, beta_end=0.012, beta_schedule="scaled_linear"
+        # )
+        # ddim_scheduler = DDIMScheduler(
+        #     num_train_timesteps=1000, beta_start=0.00085, beta_end=0.012, beta_schedule="scaled_linear"
+        # )
+        # assert ddpm_scheduler.config.num_train_timesteps == ddim_scheduler.config.num_train_timesteps
+        # set_seed(0)
+        # clean_images = [paddle.randn(shape=(4, 3, 32, 32)).clip(min=-1, max=1) for _ in range(4)]
+        # noise = [paddle.randn(shape=(4, 3, 32, 32)) for _ in range(4)]
+        # text_embeddings = [paddle.randn(shape=(4, 77, 1280)) for _ in range(4)]
+        # timesteps = [paddle.randint(0, 1000, (4,)).astype(dtype="int64") for _ in range(4)]
+        # model, optimizer = self.get_model_optimizer(resolution=32)
+        # model.train()
+        # for i in range(4):
+        #     optimizer.clear_grad()
+        #     ddpm_noisy_images = ddpm_scheduler.add_noise(clean_images[i], noise[i], timesteps[i])
+        #     ddpm_noise_pred = model(ddpm_noisy_images, timesteps[i], encoder_hidden_states=text_embeddings[i]).sample
+        #     loss = paddle.nn.functional.mse_loss(input=ddpm_noise_pred, label=noise[i])
+        #     loss.backward()
+        #     optimizer.step()
+        # del model, optimizer
+        # model, optimizer = self.get_model_optimizer(resolution=32)
+        # model.train()
+        # for i in range(4):
+        #     optimizer.clear_grad()
+        #     ddim_noisy_images = ddim_scheduler.add_noise(clean_images[i], noise[i], timesteps[i])
+        #     ddim_noise_pred = model(ddim_noisy_images, timesteps[i], encoder_hidden_states=text_embeddings[i]).sample
+        #     loss = paddle.nn.functional.mse_loss(input=ddim_noise_pred, label=noise[i])
+        #     loss.backward()
+        #     optimizer.step()
+        # del model, optimizer
+        # self.assertTrue(paddle.allclose(ddpm_noisy_images, ddim_noisy_images, atol=1e-05))
+        # self.assertTrue(paddle.allclose(ddpm_noise_pred, ddim_noise_pred, atol=1e-04))
 
     @unittest.skipIf(
         not is_ppxformers_available(),
