@@ -27,9 +27,12 @@ from ppdiffusers import (
     DiffusionPipeline,
     Mel,
     UNet2DConditionModel,
-    UNet2DModel, )
+    UNet2DModel,
+)
 from ppdiffusers.utils import slow
-from ppdiffusers.utils.testing_utils import require_paddle_gpu
+from ppdiffusers.utils.testing_utils import enable_full_determinism, require_paddle_gpu
+
+enable_full_determinism()
 
 
 class PipelineFastTests(unittest.TestCase):
@@ -48,7 +51,8 @@ class PipelineFastTests(unittest.TestCase):
             layers_per_block=2,
             block_out_channels=(128, 128),
             down_block_types=("AttnDownBlock2D", "DownBlock2D"),
-            up_block_types=("UpBlock2D", "AttnUpBlock2D"), )
+            up_block_types=("UpBlock2D", "AttnUpBlock2D"),
+        )
         return model
 
     @property
@@ -62,7 +66,8 @@ class PipelineFastTests(unittest.TestCase):
             block_out_channels=(128, 128),
             down_block_types=("CrossAttnDownBlock2D", "DownBlock2D"),
             up_block_types=("UpBlock2D", "CrossAttnUpBlock2D"),
-            cross_attention_dim=10, )
+            cross_attention_dim=10,
+        )
         return model
 
     @property
@@ -76,7 +81,8 @@ class PipelineFastTests(unittest.TestCase):
             layers_per_block=2,
             block_out_channels=(128, 128),
             down_block_types=("DownEncoderBlock2D", "DownEncoderBlock2D"),
-            up_block_types=("UpDecoderBlock2D", "UpDecoderBlock2D"), )
+            up_block_types=("UpDecoderBlock2D", "UpDecoderBlock2D"),
+        )
         unet = UNet2DModel(
             sample_size=(64, 32),
             in_channels=1,
@@ -84,14 +90,14 @@ class PipelineFastTests(unittest.TestCase):
             layers_per_block=2,
             block_out_channels=(128, 128),
             down_block_types=("AttnDownBlock2D", "DownBlock2D"),
-            up_block_types=("UpBlock2D", "AttnUpBlock2D"), )
+            up_block_types=("UpBlock2D", "AttnUpBlock2D"),
+        )
         return vqvae, unet
 
     def test_audio_diffusion(self):
-        mel = Mel()
+        mel = Mel(x_res=self.dummy_unet.config.sample_size[1], y_res=self.dummy_unet.config.sample_size[0])
         scheduler = DDPMScheduler()
-        pipe = AudioDiffusionPipeline(
-            vqvae=None, unet=self.dummy_unet, mel=mel, scheduler=scheduler)
+        pipe = AudioDiffusionPipeline(vqvae=None, unet=self.dummy_unet, mel=mel, scheduler=scheduler)
         pipe.set_progress_bar_config(disable=None)
         generator = paddle.Generator().manual_seed(42)
         output = pipe(generator=generator, steps=4)
@@ -100,54 +106,51 @@ class PipelineFastTests(unittest.TestCase):
         generator = paddle.Generator().manual_seed(42)
         output = pipe(generator=generator, steps=4, return_dict=False)
         image_from_tuple = output[0][0]
-        assert audio.shape == (1, (self.dummy_unet.config.sample_size[1] - 1) *
-                               mel.hop_length)
-        assert (image.height == self.dummy_unet.config.sample_size[0] and
-                image.width == self.dummy_unet.config.sample_size[1])
+        assert audio.shape == (1, (self.dummy_unet.config.sample_size[1] - 1) * mel.hop_length)
+        assert (
+            image.height == self.dummy_unet.config.sample_size[0]
+            and image.width == self.dummy_unet.config.sample_size[1]
+        )
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
-        image_from_tuple_slice = np.frombuffer(
-            image_from_tuple.tobytes(), dtype="uint8")[:10]
+        image_from_tuple_slice = np.frombuffer(image_from_tuple.tobytes(), dtype="uint8")[:10]
         expected_slice = np.array([0, 252, 0, 160, 144, 1, 0, 211, 99, 3])
         assert np.abs(image_slice.flatten() - expected_slice).max() == 0
-        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max(
-        ) <= 5
+        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() <= 5
+        mel = Mel(
+            x_res=self.dummy_vqvae_and_unet[0].config.sample_size[1],
+            y_res=self.dummy_vqvae_and_unet[0].config.sample_size[0],
+        )
         scheduler = DDIMScheduler()
         dummy_vqvae_and_unet = self.dummy_vqvae_and_unet
         pipe = AudioDiffusionPipeline(
-            vqvae=self.dummy_vqvae_and_unet[0],
-            unet=dummy_vqvae_and_unet[1],
-            mel=mel,
-            scheduler=scheduler)
+            vqvae=self.dummy_vqvae_and_unet[0], unet=dummy_vqvae_and_unet[1], mel=mel, scheduler=scheduler
+        )
         pipe.set_progress_bar_config(disable=None)
         np.random.seed(0)
-        raw_audio = np.random.uniform(-1, 1, (
-            (dummy_vqvae_and_unet[0].config.sample_size[1] - 1) *
-            mel.hop_length, ))
+        raw_audio = np.random.uniform(-1, 1, ((dummy_vqvae_and_unet[0].config.sample_size[1] - 1) * mel.hop_length,))
         generator = paddle.Generator().manual_seed(42)
-        output = pipe(
-            raw_audio=raw_audio, generator=generator, start_step=5, steps=10)
+        output = pipe(raw_audio=raw_audio, generator=generator, start_step=5, steps=10)
         image = output.images[0]
         assert (
             image.height == self.dummy_vqvae_and_unet[0].config.sample_size[0]
-            and
-            image.width == self.dummy_vqvae_and_unet[0].config.sample_size[1])
+            and image.width == self.dummy_vqvae_and_unet[0].config.sample_size[1]
+        )
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
-        expected_slice = np.array(
-            [128, 100, 153, 95, 92, 77, 130, 121, 81, 166])
+        expected_slice = np.array([128, 100, 153, 95, 92, 77, 130, 121, 81, 166])
+
         assert np.abs(image_slice.flatten() - expected_slice).max() <= 5
         dummy_unet_condition = self.dummy_unet_condition
         pipe = AudioDiffusionPipeline(
-            vqvae=self.dummy_vqvae_and_unet[0],
-            unet=dummy_unet_condition,
-            mel=mel,
-            scheduler=scheduler)
+            vqvae=self.dummy_vqvae_and_unet[0], unet=dummy_unet_condition, mel=mel, scheduler=scheduler
+        )
+
+        pipe.set_progress_bar_config(disable=None)
         np.random.seed(0)
         encoding = paddle.rand(shape=(1, 1, 10))
         output = pipe(generator=generator, encoding=encoding)
         image = output.images[0]
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
-        expected_slice = np.array(
-            [139, 103, 88, 105, 100, 120, 116, 99, 106, 89])
+        expected_slice = np.array([139, 103, 88, 105, 100, 120, 116, 99, 106, 89])
         assert np.abs(image_slice.flatten() - expected_slice).max() <= 5
 
 
@@ -160,18 +163,14 @@ class PipelineIntegrationTests(unittest.TestCase):
         paddle.device.cuda.empty_cache()
 
     def test_audio_diffusion(self):
-        pipe = DiffusionPipeline.from_pretrained(
-            "teticio/audio-diffusion-ddim-256")
+        pipe = DiffusionPipeline.from_pretrained("teticio/audio-diffusion-ddim-256")
         pipe.set_progress_bar_config(disable=None)
         generator = paddle.Generator().manual_seed(42)
         output = pipe(generator=generator)
         audio = output.audios[0]
         image = output.images[0]
-        assert audio.shape == (1, (pipe.unet.config.sample_size[1] - 1) *
-                               pipe.mel.hop_length)
-        assert image.height == pipe.unet.config.sample_size[
-            0] and image.width == pipe.unet.config.sample_size[1]
+        assert audio.shape == (1, (pipe.unet.config.sample_size[1] - 1) * pipe.mel.hop_length)
+        assert image.height == pipe.unet.config.sample_size[0] and image.width == pipe.unet.config.sample_size[1]
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
-        expected_slice = np.array(
-            [151, 167, 154, 144, 122, 134, 121, 105, 70, 26])
+        expected_slice = np.array([151, 167, 154, 144, 122, 134, 121, 105, 70, 26])
         assert np.abs(image_slice.flatten() - expected_slice).max() <= 5
