@@ -19,7 +19,6 @@ import unittest
 
 import numpy as np
 import paddle
-from paddlenlp.transformers import CLIPTextConfig, CLIPTextModel, CLIPTokenizer
 from PIL import Image
 
 from ppdiffusers import (
@@ -30,9 +29,10 @@ from ppdiffusers import (
     UNet2DConditionModel,
 )
 from ppdiffusers.initializer import normal_, ones_
-from ppdiffusers.pipelines.stable_diffusion.pipeline_stable_diffusion_controlnet import (
+from ppdiffusers.pipelines.controlnet.pipeline_controlnet_inpaint import (
     MultiControlNetModel,
 )
+from ppdiffusers.transformers import CLIPTextConfig, CLIPTextModel, CLIPTokenizer
 from ppdiffusers.utils import floats_tensor, load_image, load_numpy, randn_tensor, slow
 from ppdiffusers.utils.testing_utils import enable_full_determinism, require_paddle_gpu
 
@@ -239,7 +239,7 @@ class MultiControlNetInpaintPipelineFastTests(
 
     def test_save_load_optional_components(self):
         pass
-    
+
     def get_dummy_components(self):
         paddle.seed(seed=0)
         unet = UNet2DConditionModel(
@@ -395,6 +395,7 @@ class MultiControlNetInpaintPipelineFastTests(
             except NotImplementedError:
                 pass
 
+
 @slow
 @require_paddle_gpu
 class ControlNetInpaintPipelineSlowTests(unittest.TestCase):
@@ -404,9 +405,15 @@ class ControlNetInpaintPipelineSlowTests(unittest.TestCase):
         paddle.device.cuda.empty_cache()
 
     def test_canny(self):
-        controlnet = ControlNetModel.from_pretrained("lllyasviel/sd-controlnet-canny", from_diffusers=False, from_hf_hub=False)
+        controlnet = ControlNetModel.from_pretrained(
+            "lllyasviel/sd-controlnet-canny", from_diffusers=False, from_hf_hub=False
+        )
         pipe = StableDiffusionControlNetInpaintPipeline.from_pretrained(
-            "runwayml/stable-diffusion-inpainting", safety_checker=None, controlnet=controlnet, from_diffusers=False, from_hf_hub=False
+            "runwayml/stable-diffusion-inpainting",
+            safety_checker=None,
+            controlnet=controlnet,
+            from_diffusers=False,
+            from_hf_hub=False,
         )
         # pipe.enable_model_cpu_offload()
         pipe.set_progress_bar_config(disable=None)
@@ -438,9 +445,15 @@ class ControlNetInpaintPipelineSlowTests(unittest.TestCase):
         assert np.abs(expected_image - image).max() < 0.09
 
     def test_inpaint(self):
-        controlnet = ControlNetModel.from_pretrained("lllyasviel/control_v11p_sd15_inpaint", from_diffusers=False, from_hf_hub=False)
+        controlnet = ControlNetModel.from_pretrained(
+            "lllyasviel/control_v11p_sd15_inpaint", from_diffusers=False, from_hf_hub=False
+        )
         pipe = StableDiffusionControlNetInpaintPipeline.from_pretrained(
-            "runwayml/stable-diffusion-v1-5", safety_checker=None, controlnet=controlnet, from_diffusers=False, from_hf_hub=False
+            "runwayml/stable-diffusion-v1-5",
+            safety_checker=None,
+            controlnet=controlnet,
+            from_diffusers=False,
+            from_hf_hub=False,
         )
         pipe.scheduler = DDIMScheduler.from_config(pipe.scheduler.config)
         # pipe.enable_model_cpu_offload()
@@ -528,4 +541,3 @@ class ControlNetInpaintPipelineSlowTests(unittest.TestCase):
     #         gc.collect()
     #         paddle.device.cuda.empty_cache()
     #     assert np.abs(images[0] - images[1]).sum() < 0.2
-
