@@ -20,8 +20,11 @@ import numpy as np
 import paddle
 
 from ppdiffusers import DDIMScheduler, LDMSuperResolutionPipeline, UNet2DModel, VQModel
-from ppdiffusers.utils import PIL_INTERPOLATION, floats_tensor, load_image, slow
-from ppdiffusers.utils.testing_utils import enable_full_determinism, require_paddle
+from ppdiffusers.utils import PIL_INTERPOLATION, floats_tensor, load_image, slow  # noqa
+from ppdiffusers.utils.testing_utils import (  # noqa
+    enable_full_determinism,
+    require_paddle,
+)
 
 enable_full_determinism()
 
@@ -89,21 +92,3 @@ class LDMSuperResolutionPipelineFastTests(unittest.TestCase):
         init_image = self.dummy_image
         image = ldm(init_image, num_inference_steps=2, output_type="numpy").images
         assert image.shape == (1, 64, 64, 3)
-
-
-@slow
-@require_paddle
-class LDMSuperResolutionPipelineIntegrationTests(unittest.TestCase):
-    def test_inference_superresolution(self):
-        init_image = load_image(
-            "https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/vq_diffusion/teddy_bear_pool.png"
-        )
-        init_image = init_image.resize((64, 64), resample=PIL_INTERPOLATION["lanczos"])
-        ldm = LDMSuperResolutionPipeline.from_pretrained("duongna/ldm-super-resolution")
-        ldm.set_progress_bar_config(disable=None)
-        generator = paddle.Generator().manual_seed(0)
-        image = ldm(image=init_image, generator=generator, num_inference_steps=20, output_type="numpy").images
-        image_slice = image[0, -3:, -3:, -1]
-        assert image.shape == (1, 256, 256, 3)
-        expected_slice = np.array([0.7644, 0.7679, 0.7642, 0.7633, 0.7666, 0.756, 0.7425, 0.7257, 0.6907])
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.05
