@@ -19,7 +19,13 @@ import numpy as np
 import paddle
 
 from ppdiffusers import ScoreSdeVePipeline, ScoreSdeVeScheduler, UNet2DModel
-from ppdiffusers.utils.testing_utils import require_paddle, slow
+from ppdiffusers.utils.testing_utils import (
+    enable_full_determinism,
+    require_paddle,
+    slow,
+)
+
+enable_full_determinism()
 
 
 class ScoreSdeVeipelineFastTests(unittest.TestCase):
@@ -33,7 +39,8 @@ class ScoreSdeVeipelineFastTests(unittest.TestCase):
             in_channels=3,
             out_channels=3,
             down_block_types=("DownBlock2D", "AttnDownBlock2D"),
-            up_block_types=("AttnUpBlock2D", "UpBlock2D"), )
+            up_block_types=("AttnUpBlock2D", "UpBlock2D"),
+        )
         return model
 
     def test_inference(self):
@@ -42,22 +49,17 @@ class ScoreSdeVeipelineFastTests(unittest.TestCase):
         sde_ve = ScoreSdeVePipeline(unet=unet, scheduler=scheduler)
         sde_ve.set_progress_bar_config(disable=None)
         generator = paddle.Generator().manual_seed(0)
-        image = sde_ve(
-            num_inference_steps=2, output_type="numpy",
-            generator=generator).images
+        image = sde_ve(num_inference_steps=2, output_type="numpy", generator=generator).images
         generator = paddle.Generator().manual_seed(0)
-        image_from_tuple = sde_ve(
-            num_inference_steps=2,
-            output_type="numpy",
-            generator=generator,
-            return_dict=False)[0]
+        image_from_tuple = sde_ve(num_inference_steps=2, output_type="numpy", generator=generator, return_dict=False)[
+            0
+        ]
         image_slice = image[0, -3:, -3:, -1]
         image_from_tuple_slice = image_from_tuple[0, -3:, -3:, -1]
         assert image.shape == (1, 32, 32, 3)
         expected_slice = np.array([0.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0])
         assert np.abs(image_slice.flatten() - expected_slice).max() < 0.01
-        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max(
-        ) < 0.01
+        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() < 0.01
 
 
 @slow
@@ -70,9 +72,7 @@ class ScoreSdeVePipelineIntegrationTests(unittest.TestCase):
         sde_ve = ScoreSdeVePipeline(unet=model, scheduler=scheduler)
         sde_ve.set_progress_bar_config(disable=None)
         generator = paddle.Generator().manual_seed(0)
-        image = sde_ve(
-            num_inference_steps=10, output_type="numpy",
-            generator=generator).images
+        image = sde_ve(num_inference_steps=10, output_type="numpy", generator=generator).images
         image_slice = image[0, -3:, -3:, -1]
         assert image.shape == (1, 256, 256, 3)
         expected_slice = np.array([1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0])
