@@ -19,7 +19,6 @@ import unittest
 
 import numpy as np
 import paddle
-from paddlenlp.transformers import CLIPTextConfig, CLIPTextModel, CLIPTokenizer
 from PIL import Image
 
 from ppdiffusers import (
@@ -28,6 +27,7 @@ from ppdiffusers import (
     StableDiffusionInpaintPipeline,
     UNet2DConditionModel,
 )
+from ppdiffusers.transformers import CLIPTextConfig, CLIPTextModel, CLIPTokenizer
 from ppdiffusers.utils import floats_tensor, load_image
 from ppdiffusers.utils.testing_utils import (
     enable_full_determinism,
